/*
FILE:     sf-refine.C
*/
/*
VERSION:  3.11
*/
/*
DATE:     8/2/2011
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2011 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* a program to gather all the reflection data and convert it to
   cif format.   March, 2002
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "rcsb_math.h"
#include <sys/types.h>
#include <unistd.h>
#include "sf.h"
#include "util.h"

char BLOCKID[80];
int NITEM;
int NFIRST=0;
const char *CCP4= "$CCP4";  

extern CRYST cryst;
extern FILE *fw_log;

int check_mtz_label(char *file);


/***************** structure factors for final refinement ****************/ 
void write_refl_refine(FILE *fout,int argc,char **argv)
/* convert the reflection data used for final refinement to mmCIF */
{
    char prog[40], inpfile[512], I_F_type[40],  blockid[80], tmp[80];
    int i=0, n=0, format=0;
    
    for(i=1; i<argc-5; i++){
        
        if(!strcmp_case(argv[i],"-RT") && !strcmp_case(argv[i+2],"-RP") &&
           (!strcmp_case(argv[i+4],"-idat")||!strcmp_case(argv[i+4],"-icif"))){
            
            strcpy(I_F_type, argv[i+1]);
            strcpy(prog, argv[i+3]);
            strcpy(inpfile, argv[i+5]);
            
        }else if(!strcmp_case(argv[i],"-RP")&&!strcmp_case(argv[i+2],"-Rt") &&
           (!strcmp_case(argv[i+4],"-idat")||!strcmp_case(argv[i+4],"-icif"))){
            
            strcpy(prog, argv[i+1]);
            strcpy(I_F_type, argv[i+3]);
            strcpy(inpfile, argv[i+5]);
            
        }
        
    }
    
    n = cif_or_mmcif(inpfile);
    
    strcpy(blockid,"refine_set");
    strcpy(BLOCKID,"refine_set");
    
    if(n==2){ //mmCIF file
        printf("\nReflection Data File Name = %s : (mmCIF format)\n",inpfile);
        write_sf_mmcif(fout, inpfile, blockid);
        fprintf(fout, "# END OF REFLECTIONS\n\n");
        return;
    }else if(n==1){  //CIF (small molecules)
        printf("\nReflection Data File Name = %s : (CIF format)\n",inpfile);   
        cif2mmcif(fout, inpfile);
        fprintf(fout, "# END OF REFLECTIONS\n\n");
        return;
    }
    
    
    if(!strcmp_case(prog, "MTZ")){
        printf("\nReflection Data File Name = %s : (MTZ format)\n",inpfile);
        if((CCP4 = getenv("CCP4")) == NULL) {
            printf("Error! you did not set CCP4 environment!\n");
            fprintf(fw_log,"\nError! you did not set CCP4 environment!\n");
        }
        convert_mtz2mmcif(fout,inpfile, argc, argv);
        return;        
    }
    
// all other cases
 
    printf("\nReflection Data File Name = %s : (%s format)\n",inpfile, prog);

    if(!strcmp_case(prog, "CNS") || !strcmp_case(prog, "CNX") ||
       !strcmp_case(prog, "XPLOR") || !strcmp_case(prog, "X-PLOR")){
        cryst_infor(fout,blockid);
        cns2mmcif(fout, inpfile);
        
    }else if (!strcmp_case(prog, "SHELX")|| !strcmp_case(prog, "XPREP")){
        cryst_infor(fout,blockid);
        shelx2mmcif(fout,inpfile,I_F_type );
        
    }else if (!strcmp_case(prog, "TNT")){
        cryst_infor(fout,blockid);
        tnt2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(prog, "EPMR") || !strcmp_case(prog, "OTHER")) {
        cryst_infor(fout,blockid);
        epmr2mmcif(fout,inpfile, I_F_type);
        
    }else if (!strcmp_case(prog, "SAINT") ){
        write_sf_header(fout, prog,blockid, 0);
        write_saint(fout,inpfile, 1, 1, 1, 0);
        
    }else if(!strcmp_case(prog, "HKL")|| !strcmp_case(prog,"scalepack") ||
            !strcmp_case(prog, "3DSCALE") ) {
        strcpy(tmp, "HKL");
        format = data_format(inpfile, tmp);
        write_sf_header(fout, prog,blockid, format);
        write_hkl(fout, inpfile, 1, 1, 1, format);
        
    }else if(!strcmp_case(prog, "DTREK") ) {
        
        strcpy(tmp, "DTREK");
        format = data_format(inpfile, tmp);
        write_sf_header(fout, prog,blockid, format);
        write_dtrek(fout, inpfile, 1, 1, 1, format);
        
    }else if(!strcmp_case(prog, "XSCALE") ) {
        strcpy(tmp, "XSCALE");
        format = data_format(inpfile, tmp);
        write_sf_header(fout, prog,blockid, format);
        write_xscale(fout, inpfile, 1, 1, 1, format);
        
    }else {
        printf("Sorry, the input format (%s) is not supported!\n",prog );
        usage();
    }
    fprintf(fout, "#END OF REFLECTIONS\n\n");


}

void convert_mtz2mmcif(FILE *fout, char *inpfile, int argc, char **argv)
{
    char label[256], outfile[512],detail[512], **line;
    int i, j, k,m, nlab=0, nline;
    
    strcpy(detail, "");
    strcpy(label, "");
    for(i=1; i<argc; i++){    
        if(!strcmp_case(argv[i], "-DP") )break;
        if(!strcmp_case(argv[i],"-label") ){
            nlab=1;
            for(j=i+1; j<argc; j++){
                if(argv[j][0]=='-'){
                    i=j-1;
                    break;
                }
                strcat(label, argv[j]);
                strcat(label, " ");
            }
        }else if(!strcmp_case(argv[i],"-detail")){
            for(k=i+1; k<argc; k++){
                if(argv[k][0]=='-'){
                    i=k-1;
                    break;
                }
                strcat(detail, argv[k]);
                strcat(detail, " ");
            }          
        }
    }
//    printf("\nlabel=%s; detail=%s\n", label, detail  );
    
    if(nlab==0){ //automatic
        sprintf(outfile, "%s_tmp",inpfile);
        mtz2mmcif_1set(inpfile, outfile);
    }else{
        sprintf(outfile,"%s_tmp", inpfile);
        get_mtz_man(inpfile, outfile, label);
    }
    
    reformat_mtz_output(outfile);

    line=get_lines_from_file(outfile, &nline);
    
    if(nline<10){
        printf("\nError! SF conversion from file (%s) to mmcif failed!\n", inpfile);
        printf("The labels are wrong or not the correct MTZ format!\n");
        fprintf(fw_log,"\nError! SF conversion from file (%s) to mmcif failed!\n", inpfile);
        fprintf(fw_log,"The labels are wrong or not the correct MTZ format!\n");
    }else{
                
        for(m=0; m<nline; m++){
            if(!strncmp(line[m], "data_",5)){
                fprintf(fout,"%s",line[m]);
                fprintf(fout,"#\n");
                fprintf(fout,"_diffrn.id  1\n");
                fprintf(fout,"_diffrn.crystal_id  1\n");
                if(strlen(detail)<=0)strcpy(detail, "?");
                fprintf(fout,"_diffrn.details  \"%s\"\n", detail);
                fprintf(fout,"#\n");
            }else{
                fprintf(fout,"%s",line[m]);
            }
        }
        if(nline)free_memory_2d(line, nline);
    }
    delete_file(outfile);
}

void get_mtz_man(char *inpfile, char *outfile, char *label_items)
{
    char script[512], command[256] ;
    static const char *pathtmp="/tmp/";
    
    int pid=0;
    FILE *fp=NULL;

    pid=getpid();
    sprintf(script, "%smtz2various_%d.scr",pathtmp, pid);
    
    if((fp=fopen(script, "w"))==NULL){
        printf("Error, file (%s) has no written permission in directory (%s)!\n",
               script, pathtmp);
        return;
    }
    replace_char(label_items,',',' ');
    
    fprintf(fp,"#!/bin/csh \n\n");
    fprintf(fp,"# If you want the log file, replace /dev/null to test.log\n");
    
    fprintf(fp,"%s/bin/mtz2various HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, inpfile, outfile);
    fprintf(fp,"OUTPUT CIF data_mtz2cif \n");
    fprintf(fp,"LABIN %s \n", label_items);  
    fprintf(fp,"END\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup; /bin/chmod +x %s ; %s\" >/dev/null", CCP4,script,script);
    system(command);
//    delete_file(script);
    
}
    

void write_sf_mmcif(FILE *fout, char *infile, char *blockid)
/* for CIF file take all the contents */
{
    FILE *fp;
    char str[MAXL];
    
    if((fp = fopen(infile, "r"))==NULL) {        
        printf("Error! Can not open the file %s(routine: write_sf_mmcif)\n",infile);
        fprintf(fw_log, "Error! Can not open the file %s(routine: write_sf_mmcif)\n",infile);
        return;
    }
    
    while (fgets(str, sizeof str, fp) != NULL) {
        rid_of_front_space(str);
        if(!strncmp(str,"data_", 5)){
            sprintf(str,"data_%s\n", blockid);
        }
        
        fprintf(fout, "%s", str);
    }
    fclose(fp);
    
}


void cns2mmcif(FILE *fout,  char *inpfile)
/* Convert the CNS reflection data to mmCIF */
{
    char str[256],  tmp[256],  *flag_cif=NULL ;
    long  i, in=0,n=0, fn=0,  nref=0, H, K, L, nfg=0;
    float  F, sF, I, sI, pa,pb, pc, pd,ffg;
    

    FILE *finp, *ftmp;
    
    if((finp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "cns2mmcif");
    }
    
    ftmp = tmpfile();
    
    while (fgets(str, sizeof str, finp) != NULL) {
        sscanf(str,"%s",tmp);
        if(strstr_case(tmp, "DECLare")){
            if(pattern5(str,"DECLare", "NAME=IOBS", "DOM", "", "") ){
                in=1;
            }else if(pattern5(str,"DECLare", "NAME=FOBS", "DOM", "", "") ||
                     pattern5(str,"DECLare", "NAME=F_NAT", "DOM", "", "")){
                fn=1;
            }
        }else{
            if(strstr_case(str, " IOBS=")){
                in=1;
            }
            if(strstr_case(str, " FOBS=")|| strstr_case(str, " F_NAT") ){
                fn=1;
            }
            
            
            
            if(strstr_case(tmp, "INDE")){
                fprintf(ftmp, "\n");
                nref++;
            }
            n=strlen(str);
            for(i=0; i<n; i++){
                if(isspace(str[i])) str[i]=' '; 
                fprintf(ftmp, "%c", str[i]);
            }
        }
    }
    
    if(nref < 50  || (in ==0 && fn ==0) ){
        printf("Error in (cns2mmcif)! Can not find the label IOBS or FOBS in your file (%s). Or too few reflections.\n", inpfile);
        fprintf(fw_log, "Error in (cns2mmcif)! Can not find the label IOBS or FOBS in your file (%s). Or too few reflections.\n", inpfile);
        return;
    }
    
    flag_cif = new char[nref+10];
    
    rewind(ftmp);
    nfg=0;
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE")){
            
            if(strstr_case(str, "TEST=") ){
                strcpy(tmp, strstr_case(str, "TEST=")+5);
                if (sscanf(tmp,"%f", &ffg)==1){
                    (ffg>0.5) ? flag_cif[nfg] = 'f' : flag_cif[nfg] = 'o';
                    nfg++;
                }
            }else if (strstr_case(str, "FREE=") ) {
                strcpy(tmp, strstr_case(str, "FREE=")+5);
                
                if (sscanf(tmp,"%f", &ffg)==1){
                    (ffg>0.5) ? flag_cif[nfg] = 'f' : flag_cif[nfg] = 'o';
                    nfg++;
                }
            }
        }
    }
    
    if(in>0 &&  fn>0 ){ //both exist   
        if(nfg>0)
            sf_data_header(fout, 0, 1);
        else
            sf_data_header(fout, 0, 0);
    }else if(in>0 &&  fn==0 ){ //only intensity exists        
        if(nfg>0)
            sf_data_header(fout, 2, 1);
        else
            sf_data_header(fout, 2, 0);
    }else if(fn>0 &&  in==0 ){ //only amplitude exists        
        if(nfg>0)
            sf_data_header(fout, 1, 1);
        else
            sf_data_header(fout, 1, 0);

    }        
    
    i=0;
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE")){
            F=0;
            sF=0;
            I=0;
            sI=0;
            sscanf(str,"%*s %ld%ld%ld",&H,&K,&L);

            
            if((strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
                strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD="))||
               
               (strstr_case(str, "PA_") && strstr_case(str, "PB_") &&
                strstr_case(str, "PC_") && strstr_case(str, "PD_"))||
               
               (strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
                strstr_case(str, "PC=") && strstr_case(str, "PD="))
               ){
                if(i==0){
                    fprintf(fout,"_refln.pdbx_HL_A_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_B_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_C_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_D_iso\n");
                }
            }
            
            
            
            if(strstr_case(str, "FOBS=") && strstr_case(str, "IOBS=")){//both I&F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &F);

                if(!strstr(str, "SIGMA=")){
                    printf("\nError in (cns2mmcif)! the Sigma for FOBS item is not SIGMA\n");
                    printf("Please change it to SIGMA and try again.\n");
                    fprintf(fw_log, "\nError in (cns2mmcif)! the Sigma for FOBS item is not SIGMA\n");
                    fprintf(fw_log, "Please change it to SIGMA and try again.\n");
                    return;
                }
                    
                strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &sF);
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &I);
                
                if(strstr(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                }else if(strstr(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                }else{
                    printf("\nError in (cns2mmcif)! the Sigma data item is not SIGI or ISIGMA or SIGMAI.\n");
                    printf("Please change it to either of them, and try again.\n");
                    fprintf(fw_log, "\nError in (cns2mmcif)! the Sigma data item is not SIGI or ISIGMA or SIGMAI.\n");
                    fprintf(fw_log,"Please change it to either of them, and try again.\n");
                    return;
                }
                
              
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f %10.2f %8.2f %c ", 
                            H,K,L, F, sF, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f %10.2f %8.2f ", 
                            H,K,L, F, sF, I, sI);
                i++;
                
            }else if(!strstr_case(str, "FOBS=") && strstr_case(str, "IOBS=")){ //only I
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                sscanf(tmp,"%f", &I);
                
                if(strstr_case(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else 
                    sI=0;
                
                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f %c ", 
                            H,K,L, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f ", 
                            H,K,L, I, sI);
               
                i++;
                
            }else if(strstr_case(str, "FOBS=") && !strstr_case(str, "IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "SIGMA=")){
                    strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f ", 
                            H,K,L, F, sF);
                i++;
            }else if(strstr_case(str, "F_NAT") && !strstr_case(str, "IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "F_NAT")+6);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "S_NAT=")){
                    strcpy(tmp, strstr_case(str, "S_NAT=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %8.2f ", 
                            H,K,L, F, sF);
                i++;
            }
            
            if(strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
               strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD=")){
            
                strcpy(tmp, strstr_case(str, "PA_MOD=")+7);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_MOD=")+7);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_MOD=")+7);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_MOD=")+7);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f %8.3f %8.3f %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
               strstr_case(str, "PC=") && strstr_case(str, "PD=")){
            
                strcpy(tmp, strstr_case(str, "PA=")+3);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB=")+3);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC=")+3);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD=")+3);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f %8.3f %8.3f %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA_CONV=") && strstr_case(str, "PB_CONV=") &&
               strstr_case(str, "PC_CONV=") && strstr_case(str, "PD_CONV=")){
            
                strcpy(tmp, strstr_case(str, "PA_CONV=")+8);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_CONV=")+8);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_CONV=")+8);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_CONV=")+8);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f %8.3f %8.3f %8.3f \n", pa, pb, pc, pd);
                
                
            }else{
                fprintf(fout,"\n");
            }
                
        }
        
    }
    if(nref>0) delete [] flag_cif;
    if(finp) fclose(finp);
    if(ftmp) fclose(ftmp);
    
}



void check_flag_num(float *flag, long nref, const char *id, char *flag_cif)
{
    long i, nf=0, no=0, yes=0;

    for(i=0; i<nref; i++){
        if(!strcmp_case(id,"CNS")){
            if(flag[i] > 0.5 )
                nf++;
            else 
                no++;
        }else if(!strcmp_case(id,"SHELX")){
            if(flag[i] < -0.5 )
                nf++;
            else 
                no++;
        }
    }

    if(nf>10 && no >50){
//            printf("%7ld, %7ld, flag=%.3lf\n",nf, no, float(nf)/float(no));
        
        if(float(nf)/float(no) > 0.03){
            yes = 1;
        }else{
            yes = 0;
        }
    }else{
        yes = 0;
    }

    if(yes==1){ // use flag in the file
        for(i=0; i<nref; i++){
            flag_cif[i]='o';
            if(!strcmp_case(id,"CNS")){
                if(flag[i] > 0.5 )
                    flag_cif[i] = 'f';
                else 
                    flag_cif[i] = 'o';
            }else if(!strcmp_case(id,"SHELX")){
                
                if(flag[i] < -0.5 )
                    flag_cif[i] = 'f';
                else
                    flag_cif[i] = 'o';
            }
        }
        
    }else{ // generate 10% 
        

        for(i=0; i<nref; i++){
            flag_cif[i]='o';
                /*
           if(rand()%10 == 1){
                flag_cif[i]='f';
            }else{
                flag_cif[i]='o';
            }
                */
        }

    }
}
    

void shelx2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert reflection file (3I4,2F8.2,I4 for SHELX ) to CIF format
*/
{
    char  **line,  tmp[10];
    int  nline, H, K, L,n1=0,n2=0,n3=0,n4=0, n, i,len, flag=0;
    float sI , I;
    
    line=get_lines_from_file(inpfile, &nline);

    n=0;
    for(i=0; i<nline; i++){
        rid_of_end_space(line[i]);
        len=strlen(line[i]);
        if(len>30)n++;
        if(n>10){
            flag=1;
            break;
        }
    }

    if(!strcmp_case(type,"F")){
        sf_header(fout, flag, 1, 0, 0);
    }else if(!strcmp_case(type,"I")){
        sf_header(fout, flag, 0, 1, 0);
    }
    
    
    for(i=0; i<nline; i++){
        n1=0;  n2=0;  n3=0;  n4=0;
        sI = 0;
        if(strlen(line[i])<20)continue;

        strncpy(tmp, line[i]+0, 4);
        tmp[4] = '\0';
        n1 = sscanf(tmp, "%d", &H);
        
        strncpy(tmp, line[i]+4, 4);
        tmp[4] = '\0';
        n2 = sscanf(tmp, "%d", &K);

        strncpy(tmp, line[i]+8, 4);
        tmp[4] = '\0';
        n3 = sscanf(tmp, "%d", &L);

        strncpy(tmp, line[i]+12, 8);
        tmp[8] = '\0';
        n4 = sscanf(tmp, "%f", &I);
        
        strncpy(tmp, line[i]+20, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &sI);
        
        if(n1==0 ||  n2==0 ||  n3==0 || n4==0) {
            continue;
        }

        if(flag>0){ //flag exists
            strncpy(tmp, line[i]+28, 4);
            tmp[4] = '\0';
            sscanf(tmp, "%d", &n);
            
            if(n>0)
                fprintf(fout,"1 1 1 %4d %4d %4d o %10.2f %8.2f\n",  H, K, L, I, sI);
            else{
                fprintf(fout,"1 1 1 %4d %4d %4d f %10.2f %8.2f\n",  H, K, L, I, sI);
            }
        }else{
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %8.2f\n",  H, K, L, I, sI);
        }
    }
    if(nline>0) free_memory_2d(line, nline);
    
}

void tnt2mmcif(FILE *fout,  char *inpfile)
/*convert TNT reflection file (A4, 4I4,2F8.2, F8.2,F8.4 ) to CIF format
  HKL, H, K, L,  F, sigmaF, phase, FOM in each row
*/
{

    float  F, sF;
    char  str[256], data_type[10], tmp[10];
    int H, K, L,n1=0,n2=0,n3=0,n4=0;
    
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "tnt2mmcif");
    }

    
    strcpy(data_type, "F"); //assumed to be amplitude
    sf_header(fout,0,1,0,0);

    while (fgets(str, sizeof str, fp) != NULL){
        n1=0;  n2=0;  n3=0;  n4=0;
        sF = 0;
        upperstr(str);
        if(strncmp(str, "HKL ",4))continue;
        if(strlen(str)<24)continue;
        
        strncpy(tmp, str+4, 4);
        tmp[4] = '\0';
        n1 = sscanf(tmp, "%d", &H);
        
        strncpy(tmp, str+8, 4);
        tmp[4] = '\0';
        n2 = sscanf(tmp, "%d", &K);

        strncpy(tmp, str+12, 4);
        tmp[4] = '\0';
        n3 = sscanf(tmp, "%d", &L);

        strncpy(tmp, str+16, 8);
        tmp[8] = '\0';
        n4 = sscanf(tmp, "%f", &F);

        if(strlen(str)>24){ 
            strncpy(tmp, str+24, 8);
            tmp[8] = '\0';
            sscanf(tmp, "%f", &sF);
        }
        
        if(n1>0 && n2>0 && n3>0 && n4>0 ){
            fprintf(fout,"1 1 1 %4d %4d %4d  %10.1f %8.1f \n",
                    H, K, L, F, sF);
        }
    }

    fclose(fp);
    
}

void epmr2mmcif(FILE *fout,  char *inpfile,char *type)
/*convert EPMR reflection file (free format ) to CIF format
   H, K, L,  Fo, sFo in each row
*/
{

    char  str[256];
    int H, K, L, n;
    float  F, sF=0;
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "epmr2mmcif");
    }

    if(!strcmp_case(type,"F")){
        sf_header(fout, 0, 1, 0, 0);
    }else if(!strcmp_case(type,"I")){
        sf_header(fout, 0, 0, 1, 0);
    }
                   
    while (fgets(str, sizeof str, fp) != NULL){
        n=sscanf(str,"%d%d%d%f%f", &H, &K, &L, &F, &sF );
        if(n==4){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f 0.0 \n", H, K, L, F );
        }else if (n==5){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %6.2f\n", H, K, L, F, sF );
        }
    }
    
    fclose(fp);
}

void  cif2mmcif(FILE *fout, char *inpfile) 
{

    char str[512], tmp[512], inpf_tmp[512];
    char **line=NULL, **item=NULL, **lines=NULL ;
    int  index[80], i, j,n, nt, *line_index, nstr=0, nitem=0, nline=0,nline_tmp=0, start_hkl=0;
     
    FILE   *finp;

    
    if((finp=fopen(inpfile, "r")) == NULL){
        open_file_error(inpfile,"sf_cif2mmcif");
    }

    for(i=0; i<80; i++){
        index[i]=-10;
    }
    
//get # of lines    
    nline_tmp=0;
    while(fgets(str, sizeof str, finp) != NULL){ 
        nline_tmp++;
    }

    lines = new char *[nline_tmp];
    line_index = new int[nline_tmp];
   
    rewind(finp);
    nline=0;
    nt=0;
    while(fgets(str, sizeof str, finp) != NULL){
        n=sscanf(str, "%s", tmp );
        if(n<=0) continue;
        
        if(!strcmp_case(tmp,"_cell_length_a")){
            cryst.a=get_float_after_id(str, "length_a");
        }else if(!strcmp_case(tmp,"_cell_length_b")){
            cryst.b=get_float_after_id(str, "length_b");
        }else if(!strcmp_case(tmp,"_cell_length_c")){
            cryst.c=get_float_after_id(str, "length_c");
        }else if(!strcmp_case(tmp,"_cell_angle_alpha")){
            cryst.alpha=get_float_after_id(str, "angle_alpha");
        }else if(!strcmp_case(tmp,"_cell_angle_beta")){
            cryst.beta=get_float_after_id(str, "angle_beta");
        }else if(!strcmp_case(tmp,"_cell_angle_gamma")){
            cryst.gamma=get_float_after_id(str, "angle_gamma");
        }
       
        rid_of_front_end_space(str);
        nt++;
        if(strlen(str)<1 || str[0]=='#') continue;
        if(nline>10 && !strncmp(str,"data_", 5))break; //next block
        lines[nline] = new char [strlen(str)+1];
        strcpy(lines[nline],str);
        line_index[nline] = nt;
        nline++;
    }

//    printf("nline, nline_tmp= %s %.2f %.2f \n", inpfile,cryst.a, cryst.beta);
    cryst_infor(fout,BLOCKID);
   

    nitem=0;
    for(i=0; i<nline-5; i++){  //get cif items
        if(!strncmp(lines[i+0],"_refln_",7) &&
           !strncmp(lines[i+1],"_refln_",7) &&
           !strncmp(lines[i+2],"_refln_",7) &&
           !strncmp(lines[i+3],"_refln_",7)){
            
            nitem=0;
            item = new char *[80];
            for(j=i; j<nline-5 ; j++){
                if(strncmp(lines[j], "_refln_", 7)) {
                    start_hkl=j;
                    break;
                }
                item[nitem] = new char [strlen(lines[j])+1];
                sscanf(lines[j], "%s", item[nitem]);
                nitem++;
            }
            if(strncmp(lines[j], "_refln_", 7)) break; 
        }
    }
    
    if(nitem<=0) {
        printf("Error in (cif2mmcif)! The input file (%s) has no correct tokens\n", inpfile);
        fprintf(fw_log,"Error in (cif2mmcif)! The input file (%s) has no correct tokens\n", inpfile);
        if(lines && nline>0) free_memory_2d(lines, nline);
        fclose(finp);
        return;
    }
    
    get_item_index(nitem, item, index);

    n=0;
    strcpy(inpf_tmp,"");    
    for(i=start_hkl; i<nline; i++){
        strcpy(tmp,lines[i]);
        line = string_token(tmp, " " , &nstr);
        
        if(nstr != nitem ){
            printf("Error at line = %d\n", line_index[i]);
            fprintf(fw_log, "Error at line = %d\n", line_index[i]);
            continue;
        }
        n++;
        write_new_mmcif(fout, n, item, index, line);
        if(nstr>0) free_memory_2d(line, nstr);
    }
    if(item && nitem) free_memory_2d(item, nitem);
    if(lines && nline) free_memory_2d(lines, nline);
    delete [] line_index;
    
    
    fclose(finp);
}

                      
void write_new_mmcif(FILE *fout, int n, char **item, int *index, char **line)
{
    int i, k;
    
    
    if(n==1){
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        for(i=0; i<NITEM; i++){
            k=index[i];
            if( k>=0)fprintf(fout,"%s\n", item[k]);
        }
    }
    
    for(i=0; i<NITEM; i++){
        k=index[i];
        if(k >=0){
            if(i<=2){
                fprintf(fout," %3s", line[k]);
            }else if(i==9){
                fprintf(fout," %s",  line[k]);
            }else if(i==12 || i==13){
                fprintf(fout," %8s", line[k]);
            }else
                fprintf(fout," %10s", line[k]);
            
        }
        
    }
    
    fprintf(fout,"\n");
    
}

void get_item_index(int n, char **item, int index[])  

{
    int i;
    
    for(i=0; i<n; i++){
        if(!strcmp_case(item[i], "_refln_index_h")){
            strcpy(item[i],  "_refln.index_h");
            index[0]=i;
        }else if(!strcmp_case(item[i], "_refln_index_k")){
            strcpy(item[i],  "_refln.index_k");
            index[1]=i;
        }else if(!strcmp_case(item[i], "_refln_index_l")){
            strcpy(item[i],  "_refln.index_l");
            index[2]=i;
        }else if(!strcmp_case(item[i], "_refln_F_meas")){
            strcpy(item[i],  "_refln.F_meas_au");
            index[3]=i;
        }else if(!strcmp_case(item[i], "_refln_F_sigma")){
            strcpy(item[i],  "_refln.F_meas_sigma_au");
            index[4]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            
            index[5]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[6]=i;

        }else if(!strcmp_case(item[i], "_refln_F_calc")){
            strcpy(item[i],  "_refln.F_calc_au");
            index[7]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_calc")){
            strcpy(item[i],  "_refln.F_squared_calc");
            index[8]=i;
            
        }else if(!strcmp_case(item[i], "_refln_observed_status")){
            strcpy(item[i],  "_refln.status");
            index[9]=i;

        }else if(!strcmp_case(item[i], "_refln_intensity_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            index[10]=i;
        }else if(!strcmp_case(item[i], "_refln_intensity_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[11]=i;
            
        }else if(!strcmp_case(item[i], "_refln_fom")){
            strcpy(item[i],  "_refln.fom");
            index[12]=i;
            
        }else if(!strcmp_case(item[i], "_refln_phase_calc")){
            strcpy(item[i],  "_refln.phase_calc");
            index[13]=i;
        }
        
    }
    NITEM=14;
        /*
    for(i=0; i<nitem; i++){
        if(index[i]>=0)printf("cif-items %d  %d (%d) %s\n",n, i, index[i],item[index[i]]);
        printf("cif-items %d  %d (%d)\n",n, i, index[i]);
    }
        */    
}
    

void mtz2mmcif_1set( char *inpfile, char *outfile)
{
    char **label_ptr=NULL, **type_ptr=NULL, **sets_ptr=NULL;
    char label[2000], type[256], sets[256],str[512];
    int nlabel=0, ntype=0, nsets=0, i,j,min, max;
    FILE *fp=NULL;

    if((fp=fopen(inpfile, "r"))==NULL){
        open_file_error(inpfile, "mtz2mmcif_1set");
    }else{
        fgets(str, sizeof str, fp);
        if(!(str[0]=='M' && str[1]=='T' && str[2]=='Z' && str[3]==' ')){
            printf("Error! The SF file (%s) is not in MTZ format! \n",inpfile);
            fprintf(fw_log, "Error! The SF file (%s) is not in MTZ format! \n",inpfile);
            return;
        }
        fclose(fp);
    }
    
    
    get_type_label_sets(inpfile, label, type, sets);
    
    label_ptr = string_token(label," ", &nlabel);
    type_ptr = string_token(type," ", &ntype);
    sets_ptr = string_token(sets," ", &nsets);

    if(nlabel != ntype || nlabel != nsets || nsets<3){
        printf("Error in (mtz2mmcif_1set)! There is a problem with data set.\n");
        fprintf(fw_log, "Error in (mtz2mmcif_1set)! There is a problem with data set.\n");
        return;
    }
    min=90;
    max=-1;
    
    printf("items  set  type  label\n");
    for(i=0; i<nsets; i++){
        printf("   %2d   %s    %s     %s\n",
               i+1,sets_ptr[i], type_ptr[i], label_ptr[i]);
    }
    
    
    for(i=3; i<nsets; i++){
        j=atoi(sets_ptr[i]);
        if(j<min)min=j;
        if(j>max)max=j;
    }

    NFIRST=first_set_num(label_ptr, type_ptr, sets_ptr, ntype);
//    NFIRST=min;
    
    printf("Total Columns =%d, min=%d, max=%d\n",ntype, min,max);
    mtz2mmcif(inpfile,outfile, label_ptr, type_ptr, sets_ptr, ntype, 1);  
 
    check_mmcif_status(outfile);
    
    
    
}

void get_type_label_sets(char *inpfile, char *label,char *type,  char *sets)
//get data sets, labels, types
{
    char  str[2000], command[512], log[256];
    FILE *flog=NULL;

//    strcpy(log, "/tmp/mtzdmp.log"); //get log file only
    strcpy(log, "mtzdmp.log"); //get log file only
   
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;%s/etc/mtzdmp  %s  > /dev/null \"", CCP4,CCP4, inpfile);
    system(command);
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;%s/etc/mtzdmp  %s  >%s \"", CCP4,CCP4, inpfile, log);
    system(command);

    
    if((flog=fopen(log, "r"))==NULL){
        printf("Error in (get_type_label_sets)! Can not open the LOGfile (%s)\n",log);
        fprintf(fw_log, "Error in (get_type_label_sets)! Can not open the LOGfile (%s)\n",log);
        exit(0);
    }

    while(fgets(str, sizeof str, flog)!=NULL){
            
      if ( strstr(str,"* Associated datasets :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(sets,str);
                    break;
                }
            }
        }else if ( strstr(str,"* Column Labels :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(label,str);
                    break;
                }
            }
        }else if( strstr(str,"* Column Types :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(type,str);
                    break;
                }
            }
        }        
    }
    fclose(flog);
}

int first_set_num(char **label_ptr,char **type_ptr,char **sets_ptr,int ntype)
// only test F and I for the data set (return the data set number).
{
    int j, n=0;
    
    for(j=3; j<ntype-1; j++){
//        printf("Data items=%d %s  %s \n", j, type_ptr[j], label_ptr[j]);
        
        if      (type_ptr[j][0] == 'F' && type_ptr[j+1][0] == 'Q' ){
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'J' && type_ptr[j+1][0] == 'Q' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'K' && type_ptr[j+1][0] == 'M' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'L' && type_ptr[j+1][0] == 'G'  ) {
            n=atoi(sets_ptr[j]);
            break;
        }
    }
    return n;
}

void mtz2mmcif(char *filename, char *outfile, char **label_ptr,
               char **type_ptr,char **sets_ptr,int ntype, int set_num) 
// using mtz2various
{
    
    char script[512], command[512] ;
    const char *pathtmp="/tmp/";
    int pid=0, n;
    FILE *fp=NULL;

    pid=getpid();
    sprintf(script, "%smtz2various_%d.scr",pathtmp, pid);
    
    if((fp=fopen(script, "w"))==NULL){
        printf("Error! file (%s) has no written permission in directory (%s)!\n",
               script, pathtmp);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n\n");
    fprintf(fp,"# If you want the log file, replace /dev/null to test.log\n");
    
    fprintf(fp,"%s/bin/mtz2various HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, filename,outfile);
    fprintf(fp,"OUTPUT CIF data_mtz2cif \n");
    
    if(set_num==1){
        print_labin_1(fp,label_ptr,type_ptr,sets_ptr,ntype);
    }else{
        print_labin_2(fp,label_ptr,type_ptr,sets_ptr,ntype);
    }
    
    fprintf(fp,"END\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);

    n=check_mtz_label(script);
    if(n==0) {
        printf("Error! No labels are written for MTZ to mmCIF conversion\n");
        fprintf(fw_log,"Error! No labels are written for MTZ to mmCIF conversion\n");
        exit(0);
    }
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;chmod +x %s ; %s\" >/dev/null",CCP4,script, script);
    system(command);
//    printf("command=%s\n", command);
//    delete_file(script);

}

int check_mtz_label(char *file)
{
    char **line;
    int nline, i, n=1;

    line=get_lines_from_file(file, &nline);
    for(i=0; i<nline; i++){
        if(strstr(line[i], "LABIN ") && !strstr(line[i],"=")){
            n=0;
            break;
        }
    }
    if(nline) free_memory_2d(line, nline);
    return n;
}



void print_labin_1(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype)
//for first data set
{
    
    int  j,  d_set;
   
    
    d_set = NFIRST;
    
    
    fprintf(finp,"LABIN  ");  

    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //FP, sigma
               'F','Q','F', d_set, "FP", "SIGFP",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //intensity, sigma
               'J','Q','I', d_set, "I", "SIGI",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //Ano_diff, sigma
               'D','Q', 'D', d_set, "DP", "SIGDP",finp);
    

    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(+), sigma 
               'G', 'L',  'F',  d_set, "F(+)", "SIGF(+)",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(-), sigma 
               'G', 'L',  'F',  d_set, "F(-)", "SIGF(-)",finp);

              
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(+),sigma  
               'K', 'M',  'I',  d_set, "I(+)", "SIGI(+)",finp);
        
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(-),sigma  
               'K', 'M',  'I',  d_set, "I(-)", "SIGI(-)",finp);

    
    for(j=3; j<ntype-1; j++){
        if(type_ptr[j][0] == 'F' && type_ptr[j+1][0] != 'Q'){ //FC
            if(!strcmp_case(label_ptr[j], "FC") || !strcmp_case(label_ptr[j], "Fcal")){
                fprintf(finp,"FC=%s  ",label_ptr[j] );
                break;  
            }
        }
    }
    
    for(j=3; j<ntype; j++){
//        printf("set=%s : type=%s : label=%s \n",sets_ptr[j],type_ptr[j], label_ptr[j]);

        if(atoi(sets_ptr[j])!= d_set) continue; //only for this set
        
        if(type_ptr[j][0] == 'P' ){ //PHIC, PHIB
//            if(strstr_case(label_ptr[j], "PHIC") || strstr_case(label_ptr[j], "AC")){
            if(!strcmp_case(label_ptr[j], "PHIC") || !strcmp_case(label_ptr[j], "AC")){
                        
                fprintf(finp,"PHIC=%s  ",label_ptr[j] );
                
            }else if (strstr_case(label_ptr[j], "PHIB") ||
                      strstr_case(label_ptr[j], "PHIM")){
                fprintf(finp,"PHIB=%s  ",label_ptr[j] );
            }
        }else if(type_ptr[j][0] == 'A' ){ // HLA,HLB,HLC,HLD
            if(strstr_case(label_ptr[j], "HLA") ){
                fprintf(finp,"HLA=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLB") ){
                fprintf(finp,"HLB=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLC") ){
                fprintf(finp,"HLC=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLD") ){
                fprintf(finp,"HLD=%s  ",label_ptr[j] );
            }
                    
        }else if(type_ptr[j][0] == 'W' ){ //FOM
            if(strstr_case(label_ptr[j], "FOM") ){
                fprintf(finp,"FOM=%s  ",label_ptr[j] );
            }
        }
    }
    for(j=3; j<ntype; j++){
        if(type_ptr[j][0] == 'I' ){ //status
            if(strstr_case(label_ptr[j], "free") ||
               strstr_case(label_ptr[j], "flag") ||
               strstr_case(label_ptr[j], "TEST")){
                fprintf(finp,"FREE=%s ",label_ptr[j] );
                break;
            } 
        }
    }
    
    
    fprintf(finp,"\n");
}

void print_labin_2(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype)
//data after the first set.
{
    int i, max=0, d_set;
    
    max=atoi(sets_ptr[ntype-1]);
    
    for(i=NFIRST+1; i<=max; i++){ // for more data sets
        d_set=i;
        
        fprintf(finp,"LABIN  ");  

        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //FP, sigma
                   'F','Q','F', d_set, "FP", "SIGFP",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //intensity, sigma
                   'J','Q','I', d_set, "I", "SIGI",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //Ano_diff, sigma
                   'D','Q', 'D', d_set, "DP", "SIGDP",finp);
    
        
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(+), sigma 
                   'G', 'L',  'F',  d_set, "F(+)", "SIGF(+)",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(-), sigma 
                   'G', 'L',  'F',  d_set, "F(-)", "SIGF(-)",finp);
              
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(+),sigma  
                   'K', 'M',  'I',  d_set, "I(+)", "SIGI(+)",finp);
            
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(-),sigma  
                   'K', 'M',  'I',  d_set, "I(-)", "SIGI(-)",finp);
        
        fprintf(finp,"\n");
    }
}

void f_sig_pair(char **label_ptr, char **type_ptr,char **sets_ptr, int ntype,
                const char type1, const char type2, const char lab1, int nset,
                const  char *asign_lab1, const char *asign_lab2, FILE *finp)
// get data pairs (F SIGF, I SIGI, ...); ntype is total number of items
    
{
    char ch;
    int j;

    for(j=3; j<ntype-1; j++){
        
        if(atoi(sets_ptr[j])!= nset) continue; //only for this set
        
        if(strstr(asign_lab1, "(+)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(+)") && strstr_case(label_ptr[j+1], "S")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else if(strstr(asign_lab1, "(-)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(-)") && strstr_case(label_ptr[j+1], "S")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else{

            ch=label_ptr[j][0];
            
            if(type_ptr[j][0] == type1 && type_ptr[j+1][0] == type2 &&
               toupper(ch)== lab1 && strstr_case(label_ptr[j+1], "S")){ 
                        
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                break;
                
            }else if(j<ntype-2 &&
                     type_ptr[j][0] == type1 && type_ptr[j+2][0] == type2 &&
                     toupper(ch)== lab1 && strstr_case(label_ptr[j+2], "S")){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+2] );
                break;
                
            }else if(type_ptr[j][0] == type1 && type_ptr[j-1][0] == type2 && 
                     toupper(ch)== lab1 && strstr_case(label_ptr[j-1], "S")){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j-1] );
                break;
            }
            
        }
        
    }
}



void print_labin(FILE *finp,char *sffile)
{
    char  str[2000], command[512], log[256];
    char label[2000], type[256], **label_ptr=NULL, **type_ptr=NULL;
    char fp[40], sigma_fp[40],ip[40], sigma_ip[40], free[40];
    
    int nlabel=0, ntype=0, i, nf=0, nfp=0, ni=0;
   
    FILE *flog=NULL;
    
//    pid = getpid(); not the process pid
           
    strcpy(log, "/tmp/mtzdmp.log");
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;%s/etc/mtzdmp  %s  -50 >%s \"",
            CCP4, CCP4, sffile, log);
    system(command);
    
    if((flog=fopen(log, "r"))==NULL){
        printf("Error in (print_labin)! Can not open the LOGfile (%s)\n",log);
        fprintf(fw_log, "Error in (print_labin)! Can not open the LOGfile (%s)\n",log);
        exit(0);
    }

    while(fgets(str, sizeof str, flog)!=NULL){
        if (strlen(str)<20 && strstr(str,"* Column Labels :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(label,str);
                    break;
                }
            }
        }else if(strlen(str)<20 && strstr(str,"* Column Types :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(type,str);
                    break;
                }
            }
        }        
    }
    fclose(flog);
//:wq    delete_file(log);
    
    label_ptr = string_token(label," ", &nlabel);
    type_ptr = string_token(type," ", &ntype);

    if(nlabel == ntype && ntype>3){
        for(i=0; i<ntype; i++){
            if(type_ptr[i][0] == 'I'){
                if(strstr_case(label_ptr[i], "free") || strstr_case(label_ptr[i], "flag") ){
                    strcpy(free, label_ptr[i]);
                    nf=1;
                    break;
                }
            }
        }
        
        for(i=0; i<ntype-1; i++){
            if(type_ptr[i][0] == 'F' &&( type_ptr[i+1][0] == 'Q'||type_ptr[i+1][0] == 'R')){
                
                if(label_ptr[i][0]== 'F' && strstr_case(label_ptr[i+1], "SIG") ){
                    strcpy(fp, label_ptr[i]);
                    strcpy(sigma_fp, label_ptr[i+1]);
                    nfp = 1;
                    break;
                }
            }else if(type_ptr[i][0] == 'Q' && type_ptr[i+1][0] == 'F'){
                
                if(label_ptr[i+1][0]== 'F' && strstr_case(label_ptr[i], "SIG") ){
                    strcpy(fp, label_ptr[i+1]);
                    strcpy(sigma_fp, label_ptr[i]);
                    nfp=1;
                    break;
                }
            }
            
        }
        for(i=0; i<ntype-1; i++){
            if(type_ptr[i][0] == 'J' && type_ptr[i+1][0] == 'Q'){
                
                if(label_ptr[i][0]== 'I' &&  strstr_case(label_ptr[i+1], "SIG")){
                    strcpy(ip, label_ptr[i]);
                    strcpy(sigma_ip, label_ptr[i+1]);
                    ni=1;
                    break;
                }
            }else if(type_ptr[i][0] == 'Q' && type_ptr[i+1][0] == 'J'){
                
                if(label_ptr[i+1][0]== 'I' &&  strstr_case(label_ptr[i], "SIG")){
                    strcpy(ip, label_ptr[i+1]);
                    strcpy(sigma_ip, label_ptr[i]);
                    ni=1;
                    break;
                }
            }
            
        }
    }
    if(nf==0){
        if(nfp != 0){
            fprintf(finp,"LABIN FP=%s  SIGFP=%s\n",fp, sigma_fp);
        }else if(ni != 0){
            fprintf(finp,"LABIN I=%s  SIGI=%s\n",ip, sigma_ip);
        }
    }else{
        
        if(nfp != 0){
            fprintf(finp,"LABIN FP=%s  SIGFP=%s FREE=%s \n",fp, sigma_fp,free);
        }else if(ni != 0){
            fprintf(finp,"LABIN I=%s  SIGI=%s FREE=%s \n",ip, sigma_ip,free);
        }
        
    }
    
   printf("number of labels in MTZ = %d\n",nlabel);
    
    
}
void reformat_mtz_output(char *inpfile)
{
    char  **line,str[256],pdb_id[80] ;
    FILE  *fout=NULL;
    int i,j,nline;

    strcpy(pdb_id, BLOCKID);
    line=get_lines_from_file(inpfile, &nline);
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(line[i][0]=='#'){
           strcpy(line[i],"#\n");
 	}
    }
    
    fout = fopen(inpfile,"w");
    for(i=0; i<nline; i++){    
        if(!strncmp(line[i],"data_", 5)){
            
            fprintf(fout, "data_%s\n", pdb_id);
            fprintf(fout,"#\n%s\n%s\n%s\n#\n",
                    "_audit.revision_id      1_0 ",
                    "_audit.creation_date  ?",
                    "_audit.update_record    'Initial release' "
                    );
	     continue;
        }else if(i>0 && line[i][0]=='#' &&  line[i-1][0]=='#' ){
	     continue;
        }else if(!strncmp(line[i],"_audit.", 7)){
	     for(j=i; j<i+20; j++){

                 
		if(line[j][0]=='#' || !strncmp(line[j],"_cell.", 6)||
		!strncmp(line[j],"loop_", 5)){
                    i=j;
                    break;
                 }
		continue;
              }
        }
        if(strstr_space(line[i], "id     mtz2cif") ){
            replace_first_str(line[i],"mtz2cif", pdb_id, str);
            fprintf(fout, "%s", str);
        }else{
            fprintf(fout, "%s", line[i]);
        }
        
    }
    fprintf(fout,"#END OF REFLECTIONS\n");
    fclose(fout);
    if(nline>0)free_memory_2d(line, nline); 
}

void check_mmcif_status(char *outfile)
// if status is wrong, reverse it.
{
    char **lines=NULL;
    int i, nline, n=0, j=0, m,k,len;
    FILE *fout=NULL;
    
    
    lines=get_lines_from_file(outfile, &nline);
    
    for(i=0; i<nline-5; i++){ 
        if(strstr(lines[i+0],"loop_") &&
           strstr(lines[i+1],"_refln.") &&
           strstr(lines[i+2],"_refln.") ){
            n=1;
            j=i;
            
            break;
        }
        if(i>1000) break;
    }
    
    if(n<=0 || nline<100){
        printf("Error! conversion to mmcif was not successful. check the data item labels!\n");
        fprintf(fw_log, "\nError! conversion to mmcif was not successful. check the data item labels!\n");
        if(nline>0) free_memory_2d(lines, nline);
        return;
    }

    n=0;
    m=0;
    
    for(i=j; i<nline; i++){  //get cif items
        if(strstr(lines[i], " o"))
            n++;
        else if(strstr(lines[i], " f"))
            m++;
    }
    if(m>n+100) { // more flag than observed, rewrite
        fout=fopen(outfile, "w");
        
        for(i=0; i<nline; i++){
            if(strstr(lines[i+0],"loop_") &&
               strstr(lines[i+1],"_refln.") &&
               strstr(lines[i+2],"_refln.") ){
                j=i;
                break;
            }
            fprintf(fout,"%s",lines[i]);
        }

        for(i=j; i<nline; i++){
            len=strlen(lines[i]);
            
            for(k=1; k<len; k++){
                if(lines[i][k]=='o' && lines[i][k-1] ==' ')
                    lines[i][k]='f';
                else if(lines[i][k]=='f' && lines[i][k-1] ==' ')
                    lines[i][k]='o';
            }
            
            fprintf(fout,"%s",lines[i]);
        }
        fclose(fout);
    }
    
}



void  shelx_cif2mmcif(FILE *fout, char *infile) //disabled at the moment
{
    char item[40], str[256], tmp[256];
    float a;
    int n;
    FILE *fp;
    
    if((fp = fopen(infile, "r"))==NULL)
        open_file_error(infile, "shelx_cif2mmcif");

    
    while (fgets(tmp, sizeof tmp, fp) != NULL) {
        strcpy(str,tmp);
        rid_of_front_space(str);
        if(sscanf(str,"%s", item) !=1) continue;
        if(!strcmp_case(item, "_shelx_title")){
            replace_first_str(str,"_shelx_title","_shelx.title",str);
        }else if(!strcmp_case(item,"_shelx_refln_list_code")){
            replace_first_str(str,"_shelx_refln_list_code","_shelx.refln_list_code",str);
        }else if(!strcmp_case(item,"_shelx_F_calc_maximum")){
            replace_first_str(str,"_shelx_F_calc_maximum","_shelx.F_calc_maximum",str);
        }else if(!strcmp_case(item,"_exptl_crystal_F_000")){
            replace_first_str(str,"_exptl_crystal_F_000","_exptl_crystal.F_000",str);
        }else if(!strcmp_case(item,"_reflns_d_resolution_high")){
            replace_first_str(str,"_reflns_d_resolution_high","_reflns.d_resolution_high",str);
            
        }else if(!strcmp_case(item,"_symmetry_equiv_pos_as_xyz")){
            replace_first_str(str,"_symmetry_equiv_pos_as_xyz","_symmetry_equiv.pos_as_xyz",str);
            
        }else if(!strcmp_case(item,"_cell_length_a")){
            replace_first_str(str,"_cell_length_a","_cell.length_a",str);
        }else if(!strcmp_case(item,"_cell_length_b")){
            replace_first_str(str,"_cell_length_b","_cell.length_b",str);
        }else if(!strcmp_case(item,"_cell_length_c")){
            replace_first_str(str,"_cell_length_c","_cell.length_c",str);
        }else if(!strcmp_case(item,"_cell_angle_alpha")){
            replace_first_str(str,"_cell_angle_alpha","_cell.angle_alpha",str);
        }else if(!strcmp_case(item,"_cell_angle_beta")){
            replace_first_str(str,"_cell_angle_beta","_cell.angle_beta",str);
        }else if(!strcmp_case(item,"_cell_angle_gamma")){
            replace_first_str(str,"_cell_angle_gamma","_cell.angle_gamma",str);
            
        }else if(!strcmp_case(item,"_shelx_F_squared_multiplier")){
            replace_first_str(str,"_shelx_F_squared_multiplier","_shelx.F_squared_multiplier",str);
            
        }else if(!strcmp_case(item,"_refln_index_h")){
            replace_first_str(str,"_refln_index_h","_refln.index_h",str);
        }else if(!strcmp_case(item,"_refln_index_k")){
            replace_first_str(str,"_refln_index_k","_refln.index_k",str);
        }else if(!strcmp_case(item,"_refln_index_l")){
            replace_first_str(str,"_refln_index_l","_refln.index_l",str);
        }else if(!strcmp_case(item,"_refln_F_squared_calc")){
            replace_first_str(str,"_refln_F_squared_calc","_refln.F_squared_calc",str);
        }else if(!strcmp_case(item,"_refln_F_squared_meas")){
            replace_first_str(str,"_refln_F_squared_meas","_refln.F_squared_meas",str);
        }else if(!strcmp_case(item,"_refln_F_squared_sigma")){
            replace_first_str(str,"_refln_F_squared_sigma","_refln.F_squared_sigma",str);
        }else if(!strcmp_case(item,"_refln_observed_status")){
            replace_first_str(str,"_refln_observed_status","_refln.status",str);
        }
        n=sscanf(str, "%f%f%f%f%f%f", &a,&a,&a,&a,&a,&a);
        
        if(!strncmp_case(str, "_refln.index_h", 14) && n!=6){
            fprintf(fout, "_refln.wavelength_id\n");
            fprintf(fout, "_refln.crystal_id\n");
            fprintf(fout, "%s", str);   
        }else {
            if(n!=6){
                fprintf(fout, "%s", str);
            }else
                fprintf(fout, "1 1 %s", str);
        }
    }
}
