#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"
#include "harv.h"
#include "util.h"

/* the utilities for both pdb_extract and pdb_extract_sf */
FILE *fw_log;

PDBLINE p;
int check_str_isalnum(char *str);


void token_str(char *str, const char *token, long *nstr, char **line)

/* token the string and put them into line[][] */
{
    char *tokenPtr;
    long k=0,i=0;

    k= strlen(str);
    if(strstr(str, "\n")){ /*rid of \n */
        str[k-1]=' ';
    }
    
    tokenPtr = strtok(str, token);
    i = 0;
    while(tokenPtr != NULL){
        strcpy(line[i], tokenPtr);
        tokenPtr = strtok(NULL, token);
        i++;
    }
    *nstr = i;
        
}

void cifparse(char *iFile, char  *oFile, int width)
{
  string diags;
  CifFile* fobjR = NULL;
  CifParser* cifParserR = NULL;
  
//  fobjR = new CifFile(false, StringCompare::eCASE_SENSITIVE, width);
  fobjR = new CifFile(false, Char::eCASE_SENSITIVE, width);

  cifParserR = new CifParser(fobjR, fobjR->GetVerbose());

  cifParserR->Parse(iFile, diags);

  delete(cifParserR);

//  if (diags) printf(" Diagnostics [ %d  %s] \n", strlen(diags), diags );
  fobjR->Write(oFile, true);
  fobjR->Close();
  if (fobjR) delete fobjR;
  
}

char ** string_token(char *str, const char *token, int *nstr)
// use characters in 'token' to parse. Better to use character
/* token the string return **line */
{
    char *tokenPtr, **line=NULL, *tmp=NULL;
    int i=0;


    delete_return(str);
    
    tmp = new char [strlen(str) +1];
    strcpy(tmp,str);

    tokenPtr = strtok(tmp, token); //get number of token
    i = 0;
    while(tokenPtr != NULL){
        tokenPtr = strtok(NULL, token);
        i++;
    }
    delete [] tmp;

    line = new char *[i+1];
    
    tokenPtr = strtok(str, token);
    i = 0;
    while(tokenPtr != NULL){
        line[i] = new char [strlen(tokenPtr) +1];
        
        strcpy(line[i], tokenPtr);
        tokenPtr = strtok(NULL, token);
        i++;
    }
    *nstr = i;
    return (line);
    
}


char **token_pattern(char *str, const char *tok, int *ntok)
// use string (pattern) 'tok' to get ntok substrings
// better to use string as 'tok'    
{
    char *ptr, **token, *str1, *str2;
    int  i, len, len1;

    len=strlen(str);
    
    str1=new char[len+1];
    str2=new char[len+1];
    strcpy(str1, str);
    strcpy(str2, str);
    
    i=0;
    len=strlen(tok);
    while(strstr(str1,tok)){ //get number of token
        i++;
        strcpy(str1, strstr(str1,tok)+len);
    }

    
    token=new char *[i+2];
    i=0;
    while((ptr=strstr(str2,tok))){ 
        len1=ptr-str2;
        token[i] = new char [len1+1];
        strncpy(token[i], str2, len1);
        token[i][len1]='\0';
        
        i++;
        strcpy(str2, strstr(str2,tok)+len);
        
    }
    token[i] = new char [strlen(str2)+1];
    strcpy(token[i], str2);
    i++;
    *ntok=i;
//    for(i=0; i<*ntok ; i++) printf("token= %2d %s\n", i, token[i]);
    
    delete[] str1;
    delete[] str2;
    
    return token;
    
}

char **token_pattern_case(char *str, const char *tok, int *ntok)
// use string (pattern) 'tok' to get ntok substrings
// better to use string as 'tok'   (case insensitive) 
{
    char *ptr, **token, *str1, *str2;
    int  i, len, len1;

    len=strlen(str);
    
    str1=new char[len+1];
    str2=new char[len+1];
    strcpy(str1, str);
    strcpy(str2, str);
    
    i=0;
    len=strlen(tok);
    while(strstr_case(str1,tok)){ //get number of token
        i++;
        strcpy(str1, strstr_case(str1,tok)+len);
    }

    
    token=new char *[i+2];
    i=0;
    while((ptr=strstr_case(str2,tok))){ 
        len1=ptr-str2;
        token[i] = new char [len1+1];
        strncpy(token[i], str2, len1);
        token[i][len1]='\0';
        
        i++;
        strcpy(str2, strstr_case(str2,tok)+len);
        
    }
    token[i] = new char [strlen(str2)+1];
    strcpy(token[i], str2);
    i++;
    *ntok=i;
//    for(i=0; i<*ntok ; i++) printf("token= %2d %s\n", i, token[i]);
    
    delete[] str1;
    delete[] str2;
    
    return token;
    
}

char *replace_all_str(const char *str_in, const char *str1, const char *str2)
/*
  str_in is the input string 
  str1 is the string for replacement.
  str2 is the given string to replace the str1.
  return a new string!
*/
{
    char *ptr,  *str=NULL, *str_out=NULL, *tmp;
    int  i=0, n, len, len1, len2, lent=0;
    
    len1=strlen(str1)  ;
    len2=strlen(str2) + 1;
    len=strlen(str_in) + 1;
    
    str=new char[len];
    strcpy(str, str_in);
    if(!strstr(str_in, str1)){
//        printf("There is no substring (%s) in (%s)\n", str1,str_in);
        return str;
    }
    
    i=0;
    while(strstr(str,str1)){ //get number of token
        i++;
        strcpy(str, strstr(str,str1)+len1);
//    printf("Number of patterns=%d (%s), (%s), (%s)\n", i,str1, str2, str);
    }
    if(str) delete[] str;
    
    lent=len + i*(len2)+1; //total length of returning string
    str_out=new char[lent];
    str=new char[lent];
    
    strcpy(str, str_in);
    strcpy(str_out, "");
    
    while((ptr=strstr(str,str1))){ 
        n=ptr-str;
        tmp = new char [n+1];
        strncpy(tmp, str, n);
        tmp[n]='\0';
        
        strcpy(str, strstr(str, str1)+len1);
        strcpy(str_out, "" );
        strcat(str_out, tmp);
        strcat(str_out, str2);
        strcat(str_out, str);
        
        strcpy(str,str_out );
        if(tmp)delete [] tmp;
        
    }
//    printf("\nout= %2d %s\n", i, str_out);
    
    if(str)delete[] str;
    return str_out;
}


int get_string_line_patt(int start, int nline, char **line,
                         const char *end1,const char *end2,const char *end3 )
//input lines with start line number, find the pattern within start & end in
//one line or more lines and return the pattern and line contain 'end'    
{
    int i, m=start;


    for(i=start; i<nline; i++){
        if((strlen(end1) && strstr(line[i],end1)) ||
           (strlen(end2) && strstr(line[i],end2)) ||
           (strlen(end3) && strstr(line[i],end3)) ){
            m=i;
            break;
        }
    }
    return m;
}


char *get_string_patt(int n, int nline, char **line,
                      const char *start, const char *end,int *j)
//input lines with start line number, find the pattern within start & end in
//one line or more lines and return the pattern and the line num containg 'end'
// the return char striped off the multiple spaces    
{
    char *str=NULL, *substr=NULL;
    int i, m=0;

    if(strstr(line[n], start) && strstr(line[n], end)){
        substr=get_substr(line[n],start,end);
        *j=n;
        return substr;
    }
    
    for(i=n; i<nline; i++){
        m=m+strlen(line[i]);
        if(strstr(line[i],end)) break;
    }
    str=new char [m+1];
    strcpy(str,"");
    
    if(strstr(line[n], start)){
        
        for(i=n; i<nline; i++){
            
            strcat(str,line[i]);
            if(strstr(line[i],end)){
                *j=i;
                break;
            }
        }
    }
    
    replace_char(str, '\n', ' ');
    replace_char(str, '\r', ' ');
    substr=get_substr(str,start, end);
    if(str) delete [] str;
    clean_string(substr,substr);
//    printf("get_string_patt %4d %4d %4d %s %s \n", n, *j, m, start, end);
    return substr;
}

char *pdb_string_patt(int n, int nline, char **line, int pos,
                      const char *start, int *j)
//input lines with start line number, find the pattern from start until 
//meeting a new non-space string. 
// the returned string striped off the multiple spaces    
{
    char str[256], *substr=NULL, *subs=NULL;
    int i,  len=0;

    len=strlen(start);
    substr=new char[20000];
    strcpy(substr, "");
    
    for(i=n; i<nline; i++){
        strncpy(str, line[i+1]+ pos, len);
        str[len]='\0';
        strcat (substr, line[i]+ pos+len);
        if(i>n+1) strcat (substr, " ");
        
        if(is_space_line(str)>1){
            *j=i;
            break;
        }
    }
    replace_char(substr, '\n', ' ');
    replace_char(substr, '\r', ' ');
    clean_string(substr,substr);
    rid_of_front_end_space(substr);
    if(is_space_line(str)==0) strcpy(substr, "?");

    len=strlen(substr);
    subs = new char[len+1];
    strcpy(subs, substr);
    
//    printf("pdb_string_patt %4d %4d %4d %s %s \n", n, *j, pos, start, substr);
    delete [] substr;
    return subs;
}



long upperstr(char *a)
/* change to upper case, and return string length */
{
    long nlen = 0;

    while (*a) {
        nlen++;
        if (islower((int) *a))
            *a = toupper(*a);
        a++;
    }
    return nlen;
}

long lowerstr( char *a)
/* change to upper case, and return string length */
{
    long nlen = 0;

    while (*a) {
        nlen++;
        if (isupper((int) *a))
            *a = tolower(*a);
        a++;
    }
    return nlen;
}


long num_of_line(const char *inpfile)
{
    char str[512];
    long n=0;    
    FILE *fp;
    
    if((fp = fopen(inpfile, "r"))==NULL) {        
        printf("Can not open the file %s (routine:num_of_line)\n",inpfile);
        return 0;
    }
    
    n=0;    
    while (fgets(str, sizeof str, fp) != NULL) {
        n++;
    }
    fclose(fp);    
    return n;
}
    

void get_data_set_id(const char *str, const char *start, const char *end,
                     char *substr)
/* return the substr between the first start and the first end of the str */
{
    char str_start[512];
    long n1=0, len=0;
    
    
    if(!strstr_case(str, start) || !strstr_case(str, end) ) {
        strcpy(substr,"");
//        printf("There is no start (%s) or end (%s) \n", start,end);
        return;
    }
    
    len = strlen(start);
    strcpy(str_start, strstr_case(str, start)+len );
    rid_of_front_space(str_start);
    if(!strstr_case(str_start, end) ){ /*end is not after start*/
//        printf("End (%s) is not after start (%s) \n", end, start);
        strcpy(substr,"");
        return;
    }

    n1 = strstr_case(str_start, end) - str_start;
    strncpy(substr,str_start, n1);
    substr[n1]='\0';
    if(iscntrl(substr[n1-1]))substr[n1-1]='\0';
    if(iscntrl(substr[n1-2]))substr[n1-2]='\0';
         
}


void char2space(char *str, char charact)
/* subsitute the charact by a space */
{
    int i=0, len=0;

    len=strlen(str);
    for(i=0; i<len; i++){
        if(str[i]==charact) str[i]=' ';
    }
}

void rid_of_char(char *str, char ch)
/* get rid of space*/
{
    int i=0, j=0, len=0;

    len=strlen(str);
    for(i=0; i<len; i++){
        if(str[i]==ch) continue;
        str[j++]= str[i];
    }
    str[j]='\0';
    if(iscntrl(str[j-1]))str[j-1]='\0';
}

void rid_of_space(char *str)
/* get rid of space*/
{
    int i=0, j=0, len=0;

    len=strlen(str);
    for(i=0; i<len; i++){
        if(isspace(str[i])) continue;
        str[j++]= str[i];
    }
    str[j]='\0';
    if(iscntrl(str[j-1]))str[j-1]='\0';
}

void rid_of_front_end_space(char *str)
{
    int i=0, j=0, len=0, n=0;

//    if(str[0]=='?') return;
    
    
    if(!str || (len=strlen(str))==0) return;
    
    for(i=0; i<len; i++){  //front space
        if(isspace(str[i]))
            continue;
        else{
            j=i;
            break;
        }
    }
    n=0;
    for(i=j; i<len; i++){
        str[n++]=str[i];
    }
    str[n]='\0';

    if(str[0]=='\0') return;
    
    n=0;
    len=strlen(str);
    for(i=len-1; i>=0; i--){ //end space
        if(isspace(str[i]))
            n++;
        else
            break;
    }
    str[len-n]='\0';
    if((len-n-1 >= 0) && iscntrl(str[len-n-1])) str[len-n-1]='\0';
}


void rid_of_front_space(char *str)
/* get rid of first space*/
{
    int i=0, j=0, len=0, n=0;

    len=strlen(str);
    for(i=0; i<len; i++){
        if(isspace(str[i]))
            continue;
        else{
            j=i;
            break;
        }
    }
    
    for(i=j; i<len; i++){
        str[n++]=str[i];
    }
    str[n]='\0';
}

void rid_of_end_space(char *str)
/* get rid of space after last word */
{
    int i=0,  n=0, len;

    len=strlen(str);
    for(i=len-1; i>=0; i--){
        if(isspace(str[i]))
            n++;
        else
            break;
    }
    str[len-n]='\0';
    if((len-n-1 >= 0) && iscntrl(str[len-n-1]))str[len-n-1]='\0';
}



int strstr_space( const char *str, const char *str1)
/* see if str1 is in str regardless of the spaces in str and str1
 (case insensitive!)
*/
{
    int i=0, j=0, n=0, n1=0, key=0;
    char *tmp=NULL, *tmp1=NULL;
    
    tmp=(char *) calloc(strlen(str)+1, sizeof(char)+1);
    tmp1=(char *) calloc(strlen(str1)+1, sizeof(char)+1);
    
   
    strcpy(tmp, str);
    strcpy(tmp1, str1);

    n=strlen(tmp);
    n1=strlen(tmp1);

    tmp1[0] = toupper(str1[0]);
    j=1;
    for(i=1;i<n1;i++){
        if ( isspace(str1[i-1]) && isspace(str1[i]) ) continue;
        tmp1[j] = toupper(str1[i]);
        j++;
    }
    tmp1[j]='\0';

    tmp[0] = toupper(str[0]);
    j=1;
    for(i=0;i<n;i++){
        if ( isspace(str[i]) &&  isspace(str[i+1]) ) continue;
        tmp[j++] = toupper(str[i]);
    }
    tmp[j]='\0';

    if(strstr(tmp, tmp1))
        key = 1;
    else
        key = 0;

    free (tmp);
    free (tmp1);
    
    return key;
    
}


int pattern5(const char *str,const char *str1,const char *str2,
             const char *str3, const char *str4,const char *str5)
{
    char *tmp1=NULL,*tmp2=NULL,*tmp3=NULL,*tmp4=NULL,*tmp5=NULL;
    
    if(strlen(str)==0) return 0;
    
    tmp1= new char [strlen(str)+1];
    tmp2= new char [strlen(str)+1];
    tmp3= new char [strlen(str)+1];
    tmp4= new char [strlen(str)+1];
    tmp5= new char [strlen(str)+1];

    if(!str_comp(str, str1, tmp1)){
        delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
        return 0;
    }
    if(!str_comp(tmp1, str2, tmp2)){
        delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
        return 0;
    }
    if(!str_comp(tmp2, str3, tmp3)){
        delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
        return 0;
    }
    if(!str_comp(tmp3, str4, tmp4)){
        delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
        return 0;
    }
    if(!str_comp(tmp4, str5, tmp5)){
        delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
        return 0;
    }
     
    delete []tmp1; delete []tmp2; delete []tmp3; delete []tmp4; delete []tmp5;
    return 1;
}


int str_comp(const char *str, const char *str1,  char *tmp1)
{

    int n;

    if((n=strlen(str1)) <= 0){
        strcpy(tmp1, str);
        return 1;
    }
    
    if(strstr_case(str, str1)){
        strcpy(tmp1, strstr_case(str, str1) + n);
        return 1;
    }else{
        strcpy(tmp1, str);
        return 0;
    }
}


int str_cmp(const char *str, const char *str1, int n1, char *tmp1)
// compare str and str1, return new char after str1
{
    
    if(strstr_case(str, str1)){
        strcpy(tmp1 , strstr_case(str, str1) + n1);
        return 1;
    }else
        return 0;
}

int strcmp_case (const char *s1, const char *s2){
  int same =0;
  char a1, a2;

  while (s1[0]!='\0' && s2[0]!='\0' && !same) {
    a1 = tolower(s1[0]);
    a2 = tolower(s2[0]);
    if (a1!=a2){
      if (a1<a2)
          same = 1;
      else
          same = -1;
    }
    
    s1++; s2++;
  }
  if ((s1[0]!='\0'|| s2[0]!='\0')&& same==0){
      
      if (s1[0]!='\0')
          same = 1;
      else
          same = -1;
  }
  
  return same;
}

int strncmp_case (const char *s1, const char *s2, int n){

  int same =0;
  char a1, a2;
  int i=0;
  
  i=0;
  while (s1[0]!='\0' && s2[0]!='\0' && !same && i<n) {
    a1 = tolower(s1[0]);
    a2 = tolower(s2[0]);
    if (a1!=a2){
      if (a1<a2)
          same = 1;
      else
          same = -1;
    }
    
    s1++; s2++;
    i++;
  }
  if ((s1[0]!='\0'|| s2[0]!='\0')&& same==0)
    if (i<n) {
      if (s1[0]!='\0')
	same = 1;
      else
	same = -1;
    }
  return same;
}

char* strstr_case(const char *s1, const char *s2) 
{ 
    int i,j,k; 

    for(i=0;s1[i];i++) 
        for(j=i,k=0;tolower(s1[j])==tolower(s2[k]);j++,k++) 
            if(!s2[k+1]) 
                return (char*) (s1+i); 

    return NULL; 
}

void replace_char(char *str, char find, char replace)
/* replace the find by replace in the string str*/
{
    int i,  len;

    len =strlen(str);

    if(!strchr(str,find)){
//        printf("Sorry, the character (%c) is not in the string.\n(%s)\n", find ,str);
        return;
    }
    
    for(i = 0; i <len ; i++){
        if(str[i] == find ) str[i] = replace;
    }
    
}


void replace_first_str(const char *str_old, const char *str1, const char *str2,
                       char *str_new)
/*
  str_old is the old string and str_new is the new string after replace
  str1 is the first occurrence in str_old.
  str2 is the given string for replace.
*/
    
{
    char *tmp=NULL, *tmp1=NULL;
    int n1;

    if(strlen(str_old)<=0) return;
    if(strstr_case(str_old, str1) <=0){
        printf("String=%s is not in the string=%s\n", str1, str_old);
        return;
    }
    tmp= new char[strlen(str_old)+1];;
    tmp1=new char[strlen(str_old)+1];
        
    strcpy(tmp, strstr_case(str_old,str1) + strlen(str1));
    
    n1 = strstr_case(str_old,str1) - str_old;
    strncpy(tmp1, str_old, n1);
    tmp1[n1]='\0';

    sprintf(str_new, "%s%s%s", tmp1, str2,tmp);
    delete [] tmp;
    delete [] tmp1;
    
}


int is_space_line(const char *str)
// n=0, is a space line; n!=0 NOT.    
{
    int i, n=0, len;

    len = strlen(str);
    for(i=0; i<len; i++){
        if(isspace(str[i])) continue;
        n++;
    }
    return n;
}

void delete_return(char *line)
/* rid of \n */
{
    long len=0, i=0;
    
    len = strlen(line);
    for(i=0; i<len; i++){
        if(line[i]=='\n') line[i]='\0';
    }
    
    if(iscntrl(line[i-1]))line[i-1]='\0';
}

void get_string(const char *str_in, int start, int end, char *str_out)
{

    int len=0, i=0, n, stop;
    
    len = strlen(str_in);
    if(len<start || start>end){ 
        strcpy(str_out, "?");
        return ;
    }
    if(end<=len) {
        stop = end;
    }else{
        stop = len;
    }
    
//    (end<=len) ? stop = end: stop = len; //why it does not work for SGI? 
    n=0;
    for(i=start; i<=stop; i++){
        str_out[n++]=str_in[i];
    }
    str_out[n] = '\0';
    rid_of_front_end_space(str_out);
    
    if(iscntrl(str_out[n-1])) str_out[n-1]='\0';
    if(is_space_line(str_out)==0) strcpy(str_out, "?");
    
}


void get_last_string(const char *str_in, int length,  char *str_out)
        // length = length of last string
{
    char *tmp=NULL;
    long len=0,len1=0, i=0, n;

    
    len = strlen(str_in);
    tmp = new char[len+1];
    strcpy(tmp, str_in);
    
    if(len<=length){
        strcpy(str_out, "?");
//        printf("The number (length=%d ) is out of line\n", length);
        delete [] tmp;
        
        return;
    }
    
    rid_of_end_space(tmp);
    len1=strlen(tmp);
    
    n=0;
    for(i=len1-length; i<len1; i++){
        str_out[n++]=tmp[i];
    }
    str_out[n] = '\0';
    if(iscntrl(str_out[n-1])) str_out[n-1]='\0';
    if(is_space_line(str_out)==0) strcpy(str_out, "?");
    delete [] tmp;
   
}


void open_file_error(const char *inpfile, const char *message)
{
    printf("Error! Can not open the file = (%s)\n",inpfile);
    printf("Program stoped at subroutine = (%s)\n",message);
    
    fprintf(fw_log, "\nError! Can not open the file = (%s)\n",inpfile);
    fprintf(fw_log, "Program stoped at subroutine = (%s)\n",message);
    
    exit(0);
}

void write_data_message(const char *program, const char *filename)
{
    printf("\nWarning! No data was extracted from file (%s).\n", filename);
    printf("Please check the %s format!\n", program);
    printf("or send your file (%s) to 'deposit@deposit.rcsb.org'.\n\n",
           filename);
    
    fprintf(fw_log, "\nWarning! No data was extracted from file (%s).\n", filename);
    fprintf(fw_log, "Please check the %s format!\n", program);
    fprintf(fw_log, "or send your file (%s) to 'deposit@deposit.rcsb.org'.\n\n",
           filename);
}

char *get_substr(const char *str,  const char *start,  const char *end)
{
    char *str_start=NULL, *substr=NULL;
    long n1=0, len=0;
    
    len = strlen(str);
    str_start = new char [len+1];
    substr = new char [len+1];
    
    if(!strstr_case(str, start) || !strstr_case(str, end) ) {
        strcpy(substr,"?");
//        printf("There is no start (%s) or end (%s) \n", start,end);
        delete [] str_start;
        
        return substr ;
    }
    
    len = strlen(start);
    strcpy(str_start, strstr_case(str, start)+len );
    if(!strstr_case(str_start, end) ){ /*end is not after start*/
        strcpy(substr,"?");
//        printf("End (%s) is not after start (%s) \n", end, start);
        delete [] str_start;
        
        return substr;
    }else{
        
        n1 = strstr_case(str_start, end) - str_start;
//        printf("nn1=%d\n", n1);
           
        strncpy(substr,str_start, n1);
        substr[n1]='\0';
        if(iscntrl(substr[n1-1]))substr[n1-1]='\0';
        if(iscntrl(substr[n1-2]))substr[n1-2]='\0';
        delete []str_start;
        rid_of_front_space(substr);
        rid_of_end_space(substr);
        if(is_space_line(substr)==0) strcpy(substr, "?");
        return substr;
    }
}

char *get_string_after_id_number(const char *str,  const char *id,  int num)
// get string after id by the giving string number (spaced).    
{
    char *str_tmp=NULL, *substr=NULL, **line=NULL;
    int i, len=0, len_id=0, nstr=0, number;
    
    len = strlen(str);
    len_id = strlen(id);
    if (!strstr_case(str, id) || len<len_id){
        printf("The id =(%s) is not in the string (%s)\n", id, str);
        return  substr ;
    }
    
        
    str_tmp = new char [len+1];
    substr = new char [len+1];
    
    strcpy(str_tmp, strstr_case(str, id) + len_id);

    line = string_token(str_tmp," ", &nstr);
    (nstr>num)? number = num: number = nstr;
    
    strcpy(substr, "");
    for(i=0; i<number;i++){
        strcat(substr, line[i]);
        strcat(substr, " ");
    }
    delete [] line;
    delete [] str_tmp;
    rid_of_front_end_space(substr);
    return substr;
}

void  get_one_string_after_id(const char *str,  const char *id,  char *str_out)
// get string after id by the giving string number (spaced).    
{
    char *str_tmp=NULL;
    int  len=0;
    
    len = strlen(str);
    if (!strstr_case(str, id)){
        printf("The id =(%s) is not in the string (%s)\n", id, str);
        return ;
    }
        
    str_tmp = new char [len+1];
    strcpy(str_tmp, strstr_case(str, id) + strlen(id));
    if(sscanf(str_tmp, "%s", str_out) !=1){
        printf("Nothing exists after string (%s)\n", id);
        return;
    }
    delete [] str_tmp;
}

int get_int_after_id(const char *str,  const char *id)
// get string after id by the giving string number (spaced).    
{
    char *str_tmp=NULL;
    int i, len=0;
    
    len = strlen(str);
    if (!strstr_case(str, id)){
        printf("The id =(%s) is not in the string (%s)\n", id, str);
        return -9999;
    }
        
    str_tmp = new char [len+1];
    strcpy(str_tmp, strstr_case(str, id) + strlen(id));
    if(sscanf(str_tmp, "%d", &i) !=1){
//        printf("Nothing exists after string (%s)\n", id);
        delete [] str_tmp;
        return 0;
    }
    delete [] str_tmp;
    return i;
}

float get_float_after_id(const char *str,  const char *id)
// get string after id by the giving string number (spaced).    
{
    char *str_tmp=NULL;
    int len=0;
//    float a;
    double a=0;
    
    
    len = strlen(str);
    if (!strstr_case(str, id)){
        printf("The id =(%s) is not in the string (%s)\n", id, str);
        return 0;
    }
        
    str_tmp = new char [len+1];
    strcpy(str_tmp, strstr_case(str, id) + strlen(id));
    if(sscanf(str_tmp, "%lf", &a) !=1){
//        printf("Nothing exists after string (%s)\n", id);
        delete [] str_tmp;
        return  0;
    }
    delete [] str_tmp;
    
    return a;
    
}


char *get_string_after_id_length(const char *str,  const char *id,  int leng)
// get string after id by the giving length.    
{
    char *str_tmp=NULL, *substr=NULL;
    int  len=0, len_id=0,len_tmp=0,length ;
    
    len = strlen(str);
    len_id = strlen(id);
    if (!strstr_case(str, id) || len<len_id){
        printf("The id =(%s) is not in the string (%s)\n", id, str);
        return  substr ;
    }
    
        
    str_tmp = new char [len+1];
    substr = new char [len+1];
    
    strcpy(str_tmp, strstr_case(str, id) + len_id);
    len_tmp =  strlen(str_tmp);

    (len_tmp>leng)? length = leng : length =len_tmp;
    
    strncpy(substr, str_tmp, length);
    substr[length] = '\0';

    rid_of_front_end_space(substr);
    
    delete [] str_tmp;
    return substr;
}


char *get_string_by_length(const char *str,  int start,   int end)
{
    char *str_tmp=NULL, *str_out=NULL;
    int len=0,  n;

    len = strlen(str);
   
    n=end-start+1;
    if(start>end) n=1;
    
    str_tmp=new char [len+1];
    str_out = new char [n+1];
    strcpy(str_out,"?");
    
    if(len<=start || start>end){
        return  str_out;
    }

    strcpy(str_tmp, str + start);
    strncpy(str_out, str_tmp, n);
    str_out[n]='\0';

    
    delete [] str_tmp;
    if(iscntrl(str_out[n-1])) str_out[n-1]='\0';
    rid_of_end_space(str_out);
    if(is_space_line(str_out)==0) strcpy(str_out, "?");
    return str_out; 
}

void get_string_with_length(const char *str, int start,int end, char *substr)
{
    int i, j, len;
    
    len = strlen(str);
    if(len<=start || start>end){
        strcpy(substr,"?");
        printf("Warning! The given integers(start=%d; end=%d)\n",start, end);
        printf("are beyond string (%s) scope\n",str);
        return;
    }
    if(end>len) end = len;
    j=0;
    for(i=start;i<=end;i++){
        substr[j++] = str[i];
    }
    substr[j] = '\0';
}


float get_float(const char *str, const char *start,  const char *end)
{
    char *tmp=NULL;
    int n;
    float a;
    tmp=get_substr(str,start,end);
    n=sscanf(tmp, "%f", &a);
    delete [] tmp;
    
    if (n==1) return a;
    else return -99999;
}

int get_int(const char *str,  const char *start,   const char *end)
{
    int n, k;
    char *tmp=NULL;
    
    tmp = get_substr(str,start,end);
    n=sscanf(tmp, "%d", &k);
    
    delete [] tmp;
    if (n==1) return k;
    else return -99999;
}

float get_float_by_length(const char *str,  int start,   int end) 
{
    char *str_out=NULL;
    int n;
    float a;
    
    str_out=new char[strlen(str)+1];

    get_string(str, start,  end, str_out) ;

    n=sscanf(str_out, "%f", &a);
    
    delete [] str_out;
    
    if (n==1) return a;
    else return -99999.0;
}

int get_int_by_length(const char *str,  int start,   int end) 
{
    char *str_out=NULL;
    int n, a;
    
    str_out=new char[strlen(str)+1];

    get_string(str, start,  end, str_out) ;

    n=sscanf(str_out, "%d", &a);
    delete [] str_out;
    
    if (n==1) return a;
    else return -99999;
}

char *get_attribute_value(CifFile *fobj, const char *blockId,
                          const char *category, const char *attribute)
{
  ISTable *t=NULL;

  vector<string> col;
  int nRow;
  char *aString=NULL;

  Block& block = fobj->GetBlock(blockId);
  if (block.IsTablePresent(category)) {
    t=block.GetTablePtr(category);
    nRow = t->GetNumRows();

    if (nRow > 0) {
      if (t->IsColumnPresent(attribute)) {
	t->GetColumn(col, attribute);
	if (!col.empty() && (!col[0].empty())) {
	  aString = new char[col[0].size() + 1];
	  strcpy(aString,col[0].c_str());

	}
      }
    }
  }
  return(aString);
}

char **get_attribute_values(CifFile *fobj, const char *blockId,
                            const char *category,const char *attribute,
                            long *nrow)
{
  ISTable *t = NULL;
  vector<string> col;
  int nRow, ncol;
  char **strings=NULL;

//  printf("blockId,  category,  attribute =%s, %s, %s\n",  blockId,  category,  attribute);

  Block& block = fobj->GetBlock(blockId);
  if (block.IsTablePresent(category)) {
      t=block.GetTablePtr(category);
      nRow = t->GetNumRows();
//      printf("nRow= %d\n", nRow);
    
      if (nRow > 0) {
          if (t->IsColumnPresent(attribute)) {
              t->GetColumn(col, attribute);
              if (!col.empty()) {
                  strings = new char *[ col.size() + 1];
                  *nrow = col.size();
//                  printf("HERE %5d %5d  %5d\n",iCol, nRow, col.size());
                  ncol = col.size();
                  for (int i=0;  i < ncol; i++) {
                      strings[i] = new char[col[i].size() + 1];
                      strcpy(strings[i],col[i].c_str());
            
                  }
//                  nValues = col.size();
              }
          }
      }
  }
  return(strings);
}

int check_str_isalnum(char *str)
{
    int i, n=1, len;
    
    len=strlen(str);
    if(len<=0)return 0;
    for(i=0; i<len-1; i++){
        if (!isalnum(str[i])) {
            n=0;
            break;
        }
    }
    return n;
}

            
    

void parse_pdbline(char *str)
{
    char tmp[80];

    strncpy(tmp, str+12, 4); 
    tmp[4] = '\0';
    strcpy(p.atomname, tmp); //atom name
    
    p.alt = str[16];  //alter id
    
    strncpy(tmp, str+17, 3);
    tmp[3]='\0';
    strcpy(p.resname, tmp); //res name
    
    p.chain=str[21]; //chain id
    
    strncpy(tmp, str+22, 4);
    tmp[4]='\0';
    strcpy(p.reseq, tmp); //res number
    
    p.icode=str[26]; //insert code

    strncpy(tmp, str + 30, 8);
    tmp[8] = '\0';
    p.x=atof(tmp);
            
    strncpy(tmp, str + 38, 8);
    tmp[8] = '\0';
    p.y=atof(tmp);
            
    strncpy(tmp, str + 46, 8);
    tmp[8] = '\0';
    p.z=atof(tmp);
    
    strncpy(tmp, str + 54, 6);
    tmp[6] = '\0';
    p.occ=atof(tmp);
    
    strncpy(tmp, str + 60, 6);
    tmp[6] = '\0';
    p.b=atof(tmp);
}


char * check_pdb(char *inpfile)
{
    char **line, *infor;
    int i,j, nline, id=0,  len;
    int nchain=0, natm=0, nsiga=0, nanis=0,nsigu=0, nres=0, nocc=0;
    
    
    static const char *residues30[]={
        "LEU","PRO","ALA","VAL","GLY","SER","ASN","GLN","ASP","GLU","MET","TRP","PHE",
        "ILE","THR","CYS","TYR","HIS","LYS","ARG",
        "ADE","GUA","CYT","THY","URI","  A","  G","  T","  U","  C"
    };

    infor=new char [256];
    
    line=get_lines_from_file(inpfile, &nline);

    for (i=0;  i < nline; i++) {
        len = strlen(line[i]);
        
        if ((!strncmp(line[i], "ATOM  ", 6)|| !strncmp(line[i], "HETATM ", 7))
            && len>54 ) {
            natm++;
            parse_pdbline(line[i]);

            if(p.occ<0.99) nocc++; //check if occupancy <1
            
            
            id=0;
            for(j=0;j<30;j++){ //check if standard residue
                if(!strcmp(residues30[j], p.resname)){
                    id=1;
                    break;
                }
            }
            if(id>0) nres++;
            
            if(p.chain !=' ') nchain++;
            
        
        }else if(!strncmp(line[i], "SIGATM ", 7)){
            nsiga++;
        }else if(!strncmp(line[i], "ANISOU ", 7)){
            nanis++;
        }else if(!strncmp(line[i], "SIGUIJ ", 7)){
            nsigu++;
        }
    }
    strcpy(infor,"");
    if(natm>0){
        strcat(infor, "ATOM  ");
    }
    if(nchain>0){
        strcat(infor, "CHAIN  ");
    }
    if(nocc>0){
        strcat(infor, "OCC  ");
    }
    if(nres>0){
        strcat(infor, "PDBRES  ");
    }
    if(nsiga>0){
        strcat(infor, "SIGATM  ");
    }
    if(nanis>0){
        strcat(infor, "ANISOU  ");
    }
    if(nsigu>0){
        strcat(infor, "SIGUIJ  ");
    }
        
        
    
    if( nchain<1 ){
        printf("\nWarning! The PDB file (%s) does not have the right Chain-ID.\n",inpfile);
        fprintf(fw_log,"\nWarning! The PDB file (%s) does not have the right Chain-ID.\n",inpfile);
    }
    
    return infor;
}


int is_protein_na_residue(int nres, char *res, const char **residues)
// >0 is the standard residue
{    
    int id=0, i;
 
    for(i=0;i<nres;i++){
     
        if(!strcmp_case(residues[i], res)){
            id=1;
            break;
        }
    }
    return id;
}


int  check_category(const char *str, char *category, char *item)
// get possible category between _ and . (no space in between)     
{
    int  n=0, i=0;
    char  *tmp=NULL;

    strcpy(category, "");
    strcpy(item, "");
    
    tmp = new char[strlen(str)+1];
    
    n=sscanf(str,"%s",tmp);
        
    if(n!=1 || tmp[0] != '_' || !strchr(tmp,'.')
       || strlen(tmp)<4 || strlen(tmp)>150) {
        strcpy(category, "");
        strcpy(item, "");
        delete []tmp;
        return 0;
    }
        
    strcpy(item, strchr(tmp,'.')+1);
    
    for(i=0; tmp[i]!='.'; i++){
        category[i]=tmp[i];
    }
    category[i] = '\0';
        
    if((strlen(category)<2 || strchr(category,'.')) ||
      (strlen(item)<1 || strchr(item,'.')) ) {
        strcpy(category, "");
        strcpy(item, "");
        delete []tmp;
        return 0;
    }

    delete []tmp;

    return 1;
    
}


void  convert_cif2rcsbcif(char *inpfile, char *inpfile_new, const char *prog)
{
    char **line, str[512], tmp[256], tmp1[256], item[256];
    int i,n, nline;
    FILE *fw;
    line=get_lines_from_file(inpfile, &nline);

    fw=fopen(inpfile_new,"w");
    
    if(!strcmp_case(prog,"scala")){
        for(i=0;i<nline;i++){
            strcpy(str, line[i]);
            
            strcpy(tmp,"");
            n=sscanf(line[i],"%s %s", tmp, tmp1);
            if(strstr_case(tmp, "_diffrn_reflns.") && n==2){
                strcpy(item,strstr_case(tmp,"flns."+4));
                
                if(!strcmp_case(item,".d_res_low")){
                    sprintf(str,"_reflns.d_resolution_low  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".d_res_high")){
                    sprintf(str,"_reflns.d_resolution_high  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".number_measured_all")){
                    sprintf(str,"_reflns.pdbx_number_measured_all  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".number_unique_all")){
                    sprintf(str,"_reflns.number_all  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".number_centric_all")){
                    sprintf(str,"_reflns.pdbx_number_centric  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".number_anomalous_all")){
                    sprintf(str,"_reflns.pdbx_number_anomalous  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".Rmerge_I_anomalous_all")){
                    sprintf(str,"_reflns.pdbx_Rmerge_I_anomalous_all  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".anom_diff_percent_meas")){
                    sprintf(str,"_reflns.pdbx_anom_diff_percent_meas  %s\n",tmp1);

                    
                }else if(!strcmp_case(item,".Rmerge_I_all")){
                    sprintf(str,"_reflns.pdbx_Rmerge_I_obs  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".meanI_over_sigI_all")){
                    sprintf(str,"_reflns.pdbx_netI_over_av_sigmaI  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".meanI_over_sd_all")){
                    sprintf(str,"_reflns.pdbx_netI_over_sigmaI  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".percent_possible_all")){
                    sprintf(str,"_reflns.percent_possible_obs  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".multiplicity")){
                    sprintf(str,"_reflns.pdbx_redundancy  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".Rmeas_mean")){
                    sprintf(str,"_reflns.pdbx_Rmeas_mean   %s\n",tmp1);

                    
                }else if(!strcmp_case(item,".PCV_mean")){
                    sprintf(str,"_reflns.pdbx_PCV_mean  %s\n",tmp1);
                    
                }else if(!strcmp_case(item,".Rmeas")){
                    sprintf(str,"_reflns.pdbx_Rrim_I_all  %s\n",tmp1);

                    
                }else {
                    sprintf(str,"_reflns%s  %s\n",item, tmp1);
                }
                
            }else if (!strcmp_case(tmp, "_reflns_shell.Rmerge_I_all") && n==1){
                strcpy(str,"_reflns_shell.Rmerge_I_obs\n");
                
            }else if (!strcmp_case(tmp, "_reflns_shell.multiplicity") && n==1){
                strcpy(str,"_reflns_shell.pdbx_redundancy\n");
            }
            
            
            fprintf(fw,"%s", str);
        }
        
    }else{
        for(i=0;i<nline;i++) fprintf(fw,"%s", line[i]);
    }
    
        
    if(nline)free_memory_2d(line, nline);
    fclose(fw);
}


void change_file_name(const char *inpfile, char *outfile)
{
    char str[512];
    FILE *fp=NULL, *fout=NULL;

    if((fp=fopen(inpfile, "r"))==NULL){
        open_file_error(inpfile, "change_file_name");
    }
    fout=fopen(outfile, "w");
    
    while(fgets(str, sizeof str,fp)!=NULL){
        fprintf(fout, "%s", str);
    }
    fclose(fp);
    fclose(fout);
    
}

    

void delete_file(const char *file)
{
    char command[512];
    
    strcpy(command,"");
    
    if(getenv("HOMEDRIVE") != NULL) {/* PC! */
        
        strcpy(command, "del  ");
        strcat(command, file);
        system(command);
    }else if(getenv("HOME") != NULL) {/* Unix */
        sprintf(command, "/bin/csh -c \" rm -f %s\"", file);
        system(command);
    }else{
        sprintf(command, "/bin/csh -c \" rm -f %s\"", file);
        system(command);
    }
    
    strcpy(command, "rm -f  ");
    strcat(command, file);
    system(command);
}
    

char *make_string(const char *str1, const char *str2, const char *str3)
{
    int m;
    char *string=NULL;
    

    m = strlen(str1) + strlen(str2) + strlen(str3);
    string = new char[m+1];
    strcpy(string, "");
    strcat(string, str1);
    strcat(string, str2);
    strcat(string, str3);

    return string;
}


void clean_string(const char *str1,  char *str2)
//clean multiple space
{
    
    int i, len, n=0;
    
    len=strlen(str1);
    if(len<2){
        strcpy(str2,str1);
        return;
    }
    str2[0]=str1[0];
    n++;
    for (i=1; i<len; i++){
        if(isspace(str1[i]) && isspace(str1[i-1]))continue;
        str2[n]=str1[i];
        n++;
    }
    str2[n]='\0';
    
}

void array_sort(char ** str, int max)
 /* str 2D array; max array length*/
{
    int i, j;
    char *tmp;
    

    for(j = 0; j <max-1 ; j++){
        for (i=j+1; i<max; i++){
            if(strcmp(str[i], str[j])>0){
                tmp = str[j];
                str[j]= str[i];
                str[i]= tmp;
            }
        }
    }
    
}

void larray_sort(int *array, int max, int key)
 /* str 2D array; max array length: speed is low for big max.
    key=0; sort from lower to higher
    key=1; sort from higher to  lower
  */
{
    int i, j, tmp;
    

    for(j = 0; j <max-1 ; j++){
        for (i=j+1; i<max; i++){
            if(key==0){
                if(array[i]<array[j]){
                    tmp = array[j];
                    array[j]= array[i];
                    array[i]= tmp;
                }
            }else {
                if(array[i]> array[j]){
                    tmp = array[j];
                    array[j]= array[i];
                    array[i]= tmp;
                }
            }
            
        }
    }
    
}

     
void print_mmcif_head(FILE *fout, const char *category, const char **items, int nitems)
{
    int i;
    
    fprintf(fout, "#\nloop_\n");
    for(i=0; i<nitems;i++) {
        fprintf(fout, "%s.%s\n", category, items[i]);
//        printf( "%s.%s\n", category, items[i]);
    }
}

void write_mmcif_head(FILE *fout, const char *cate, const char **items)
{
    int i;
    
    fprintf(fout, "#\nloop_\n");
    
    for(i=0; i<1000;i++) {
        if(!strcmp(items[i], "END_CATE")) break;
        fprintf(fout, "%s.%s\n", cate, items[i]);
    }
}

char **get_lines_from_file(char *inpfile, int *nline)
        //there is no space between lines
{
    char str[50000], **all_line=NULL;
    long m;
    FILE *fp=NULL;
    
    if((fp=fopen(inpfile, "r"))==NULL){
        printf("Can not open file (%s) in get_lines_from_file\n", inpfile);
        *nline=0;
        return NULL;
    }
    
    
    m=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if(is_space_line(str)<=0)continue;
        m++;
    }
    rewind(fp);
    all_line=new char *[m+1];

    m=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if(is_space_line(str)<=0)continue;
         all_line[m] = new char [strlen(str) +1];
        strcpy(all_line[m],str);
        m++;
    }
    *nline =m;
    fclose(fp);
    return (all_line);
}

char **set_memory_2d( int nptr, int length)
{
    char **var=NULL;
    int i;

    if(nptr<=0 || length<=0) return (var);
    
    var = new char *[nptr];
    if(var == NULL){
        printf("Too many points (%d) were assigned\n",nptr);
        return(NULL);
    }
    for(i=0; i<nptr; i++){
        var[i] =  new char[length];
            
        if(var[i] == NULL){
            printf("Too many points (nptr=%d; length=%d) were assigned\n",nptr,i);
            return(NULL);
        }
            
    }
    
    return (var);
    
}


void free_memory_2d(char **var, int nptr)
{
    int i;
    if(nptr<=0) return;
    
    for(i=0; i<nptr-1; i++){
        if(var[i]) delete [] var[i];
    }
    if (var) delete [] var;
}

void file_handle(char *inpfile,char *outfile)
/* uncompress *.Z or gunzip *.gz file.
   If the extension is *.Z or *.gz, the outfile is still generated without ext.
 */    
{
    
    char *pchar;
    char command[512];
    int len=0, i=0;

    pchar = strrchr(inpfile, '.');
    if (pchar == NULL){
        strcpy(outfile, inpfile);
        return;
    }else {
        i = pchar - inpfile;
        len = strlen(inpfile) - i;
        
        if(len != 2 &&  len != 3  ) {
            strcpy(outfile, inpfile);
            return;
        }else if(len==2){
            if(inpfile[i+1] == 'Z'){
                sprintf(command,"/usr/bin/uncompress -f %s", inpfile);
                system(command);
                strncpy(outfile, inpfile, i);
                outfile[i] = '\0';
                return;
            }else{    
                strcpy(outfile, inpfile);
                return;
            }
        }else if(len==3){
           if(inpfile[i+1] == 'g' && inpfile[i+2]=='z'){
                sprintf(command,"/usr/bin/gunzip -f %s", inpfile);
                system(command);
                strncpy(outfile, inpfile, i);
                outfile[i] = '\0';
                return;
            }else{    
                strcpy(outfile, inpfile);
                return;
            }
            
        }
    }
}
  
int space_group_convertor(char *sg_name_inp, char *sg_name_out)
// convert  space group from non-standard to standard 
{
    int i, n=-1, nsg=92;
    char sg_inp_tmp[20],sg_out_tmp[20], symmetry[20];
    
    static const char *symmetry92[]=
        {
            
      "A 1        ",
      "A 1 2 1    ",
      "A 2        ", 
      "B 1 1 2    ", 
      "B 2        ", 
      "B 2 21 2   ",   
      "C 2        ",
      "C 1 2 1    ",
      "C 21       ",
      "C 1 21 1   ",  
      "C 2(A 112) ",   
      "C 2 2 2    ",   
      "C 2 2 21   ",   
      "C 4 21 2   ",
      "F 2 2 2    ",  
      "F 2 3      ",
      "F 4 2 2    ",
      "F 4 3 2    ",
      "F 41 3 2   ",
      "I 1 2 1    ",
      "I 1 21 1   ",
      "I 2        ",
      "I 2 2 2    ",   
      "I 2 3      ",
      "I 21       ",
      "I 21 3     ",
      "I 4        ",
      "I 21 21 21 ",
      "I 4 2 2    ",
      "I 4 3 2    ",
      "I 41       ",
      "I 41 2 2   ",
      "I 41 3 2   ",
      "P 1        ",    
      "P 1-       ",
      "P 2        ",  
      "P 1 2 1    ",
      "P 1 1 2    ",
      "P 2 2 2    ", 
      "P 2 3      ",
      "P 2 2 21   ",  
      "P 2 21 21  ",
      "P 21       ",  
      "P 1 21 1   ",
      "P 1 1 21   ",
      "P 21(C)    ", 
      "P 21 2 21  ",
      "P 21 3     ",
      "P 21 21 2  ",
      "P 21 21 2 A ",
      "P 21 21 21 ",
      "P 3        ",
      "P 3 1 2    ",
      "P 3 2 1    ",
      "P 31       ",
      "P 31 1 2   ",
      "P 31 2 1   ",
      "P 32       ",
      "P 32 1 2   ",
      "P 32 2 1   ",
      "P 4        ",
      "P 4 2 2    ",
      "P 4 3 2    ",
      "P 4 21 2   ",
      "P 41       ",
      "P 41 2 2   ",
      "P 41 3 2   ",
      "P 41 21 2  ",
      "P 42       ",
      "P 42 2 2   ",
      "P 42 3 2   ",
      "P 42 21 2  ",
      "P 43       ",
      "P 43 2 2   ",
      "P 43 3 2   ",
      "P 43 21 2  ",
      "P 6        ",
      "P 6 2 2    ",
      "P 61       ",
      "P 61 2 2   ",
      "P 62       ",
      "P 62 2 2   ",
      "P 63       ",
      "P 63 2 2   ",
      "P 64       ",
      "P 64 2 2   ",
      "P 65       ",
      "P 65 2 2   ",
      "H 3        ",
      "R 3        ",
      "H 3 2      ",
      "R 3 2      "   
        };
    
    
    strcpy(sg_inp_tmp, sg_name_inp);
    if(strchr(sg_inp_tmp,'(')) rid_of_char(sg_inp_tmp, '(');
    if(strchr(sg_inp_tmp,')')) rid_of_char(sg_inp_tmp, ')');
    rid_of_space(sg_inp_tmp);
           
    for (i = 0; i < nsg ; i++) {
        strcpy(symmetry,symmetry92[i]);
        
        strcpy(sg_out_tmp, symmetry);
        rid_of_space(sg_out_tmp);
           
        if (!strcmp_case(sg_inp_tmp, sg_out_tmp)){
            rid_of_front_end_space(symmetry);
            
            strcpy(sg_name_out, symmetry);
            n = i;
            break;
        }
    }
    
    if(n<0){ //space group name wrong!
        strcpy(sg_name_out, sg_name_inp);
        printf("Warning: the space group (%s) can not be converted to the standard.\n", sg_name_inp);
        fprintf(fw_log,"Warning: the space group (%s) can not be converted to the standard.\n", sg_name_inp);
        
        return n;
    }
    
    return n;
       
}



FILE *get_temp_file( char *inpfile)
//     give an inpfile to get a tempfile
{
    int j=0, ch;
    FILE *fp=NULL, *ftmp;

    
    if((fp=fopen(inpfile,"r"))==NULL){
        
        open_file_error(inpfile,"get_temp_file");
    }
    
    ftmp = tmpfile();
    
    for(j=0; (ch=getc(fp)) != EOF; j++){
        fprintf(ftmp,"%c", ch);
    }

    fclose(fp);
    rewind(ftmp);
    return ftmp; 
    
}

void rid_of_file_path_unix(char *str)
// rid of full path    
{
    char *str_tmp=NULL;
    int len=0;


    if(!strstr(str,"/") || !str)return;
    
    len=strlen(str);
    str_tmp = new char[len+10];
    strcpy(str_tmp,  strrchr(str, '/')+1);
    strcpy(str, str_tmp);
    
    if(str_tmp) delete str_tmp;
    
}

void clean_file_path(char *str, char *path, char *str_new)
// str: with full path; str_new: no path.  
{
    int  n;

    if(!strstr(str,"/") || !str)return;
    
    strcpy(str_new,  strrchr(str, '/')+1);
    n=strrchr(str, '/')-str;
    strncpy(path, str,n);
    path[n]='\0';
    
}


void dos2unix(char *file)
// rid-of  ^M characters in dos
{
    
    char command[512], file_out[512];
    const char *str=" \"\\012\\015\"  \"\\012\\012\" ";
    
    
    FILE *fp=NULL;
    
    
    strcpy(command,"");
    sprintf(file_out, "%s_dos2unix", file);

// here,do not use ' /bin/csh -c " command " for tr   
    sprintf(command, " tr -s  %s < %s >  %s  ", str, file, file_out);
    system(command);
    
    if((fp=fopen(file_out, "r"))!=NULL){        
        strcpy(command,"");
        sprintf(command, "/bin/csh -c \" mv -f %s  %s \"", file_out, file);
        system(command);
    }
    fclose(fp);
    
}
   
    

int is_cif(const char *inpfile)
/* all the possible categories are seperated by a space */
{
    char str[512], tmp[512], category[100], item[100];
    int ncat=0,  n;
    FILE *fp;

    
    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "is_cif");
    }

    while(fgets(str, sizeof str, fp)!= NULL){
        rid_of_front_end_space(str);
        
        n=sscanf(str,"%s",tmp);
        
        if(n!=1 || tmp[0] != '_' || !strchr(tmp,'.')
           || strlen(tmp)<=1 || strlen(tmp)>150) continue;
        
        strcpy(item, strchr(tmp,'.')+1);
        get_data_set_id(tmp,"_", ".", category);
        
        if(strlen(category)<1 || strchr(category,'.')) continue;
        if(strlen(item)<1 || strchr(item,'.')) continue;

        ncat++;
        if(ncat>60) break;
    
    }
    
    
    fclose(fp);
    return ncat;
}


int is_pdb(const char *inpfile, int *chain)
{
    char str[256],atom[5], res[5], x[10],y[10],z[10], juck[10] ;
    int   len, nchain=0, record=0, ncif=0;
    float a;
    FILE *fp;
    
        /*
    static const char *residues36[]={
        "LEU","PRO","ALA","VAL","GLY","SER","ASN","GLN","ASP","GLU","MET","TRP","PHE",
        "ILE","THR","CYS","TYR","HIS","LYS","ARG",
        "ADE","GUA","CYT","THY","URI",
        "  A","  G","  T","  U","  I","  C"," DA"," DG"," DT"," DC"," DI"
    };
        */
    
    *chain=1;
    
    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "is_pdb");
    }

    if(is_cif(inpfile)>8){
        printf("\nError! The coordinate file (%s) is not in PDB format (but in mmCIF).\n",inpfile);
        fprintf(fw_log, "\nError! The coordinate file (%s) is not in PDB format (but in mmCIF).\n",inpfile);
        return 0;
    }
    
    ncif=0;
    while(fgets(str, sizeof str, fp)!= NULL){
        rid_of_front_space(str);
        if(!strncmp_case(str, "_atom_site.", 11)){
            if(ncif++>6){
                printf("\nError! File (%s) is not in PDB format (looks like in mmCIF).\n",inpfile);
                fprintf(fw_log, "\nError! File (%s) is not in PDB format (looks like in mmCIF).\n",inpfile );
                fclose(fp);
                return 0;
            }
        }
        
        
        len = strlen(str);
        
        if ((!strncmp_case(str, "ATOM  ", 6)|| !strncmp_case(str, "HETATM", 6))
            && len>54 ) {
            
            strncpy(atom, str+12,4);
            atom[4]='\0';
            
            strncpy(res, str+17,3);
            res[3]='\0';
//            key=is_protein_na_residue(36, res, residues36);

            strncpy(x, str+30, 8);
            x[8]='\0';
            strncpy(y, str+38, 8);
            y[8]='\0';
            strncpy(z, str+46, 8);
            z[8]='\0';

            if(sscanf(x, "%f", &a)==1 && sscanf(y, "%f", &a)==1 &&
               sscanf(z, "%f", &a)==1 && sscanf(atom, "%s", juck)==1 &&
               sscanf(res, "%s", juck)==1){
                record++;
                if(isalnum(str[21])) nchain++;                
            }
            
        }
    }
    fclose(fp);
    
    
    if( nchain<1 ){
        printf("\nError! The PDB file (%s) does not have correct Chain-ID.\n",inpfile);
        fprintf(fw_log, "\nError! The PDB file (%s) does not have correct Chain-ID.\n",inpfile);
        printf("Each copy of molecules must be assigned a Chain-ID (using A~Z,0~9)\n");
        
        printf("If one of the following conditions is satisfied.\n");
        printf(" 1. only one copy of molecule in the file. \n");
        printf(" 2. several copies in the file, but each copy is separated by TER. \n");
        printf(" 3. each copy is separated by MODEL/ENDMDL.\n");
        printf("you can use the command to generate chain ID.\n");
        printf("\nextract -chain pdb_file_name\n\n");
        *chain=0;
    }
    
    return record;
}

int non_ascii(char *infile)
{
    int c, id=0;
    FILE *fp= NULL;
    
    if((fp = fopen(infile, "r"))==NULL){
        printf("Error! Can not open file (%s) in non_ascii.\n", infile);
        return 0;
    }
    while((c=getc(fp))!=EOF){
        if (c>127) {
            id=c;
            break;
        }
    }

    fclose(fp);
//    printf("file-ID=%d %s\n", id, infile);
    
    return id;
}

    
void clean_file(char *infile, char *outfile)
{
/*
 9  horizontal tab
 10 line feed
 13 carriage return
 32- 47:   ! " # $ % & ' ( ) * + , - . / (32 is a blank)
 48- 57: 0 ... 9
 58- 64: : ; < = > ? @
 65- 90: A ... Z
 91- 96: [ \ ] ^ _ `
 97-122: a ... z
 123-126: { | } ~
*/
    char *temp=NULL, command[512], tmpfile[256], **line1, **line2;
    int c, i, id=0, nline1, nline2;
    FILE *fp= NULL, *fo = NULL;

    

    sprintf(tmpfile, "%s__tmp", infile);
    temp = getenv("PDB_EXTRACT");
    sprintf(command,"/bin/csh -c \"python  %s/pdb-extract-data/to_ascii.py %s %s\"", temp, infile,tmpfile );
    system(command);
//    printf("command=%s <p>\n", command);
    
    line1=get_lines_from_file(tmpfile, &nline1);
    line2=get_lines_from_file(infile, &nline2);
    
    if(nline1 == nline2 && nline1>1){ //python conversion successful
        sprintf(command,"/bin/csh -c \"mv -f %s %s\"", tmpfile, outfile);
        system(command);
    }else{
        delete_file(tmpfile);
        if((fp = fopen(infile, "r"))==NULL){
            printf("Error! can not open file (%s) in clean_file.\n", infile);
            return;
        }
    
        fo = fopen(outfile, "w");
        c = getc(fp);
        while (!feof(fp)) {
            i = getc(fp);
            if (c == 13 && i != 10) c = 10;
            if ((c > 31 && c < 127 )|| c == 10)
                putc(c, fo);
            else if (c == 9 || c == 127)
                putc(' ', fo);
            else if ( c >127){
                id=1;
                putc('?', fo);
            }
            c = i;
        }
        fclose (fp);
        fclose (fo);
        if(id==1){
            printf("Warning! The file (%s) contains non-ASCII charactors. \n", infile);
            printf("They have been replaced by '?'. Please check and give proper ASCII  charactor\n\n");
        }
    }
    if(nline1)free_memory_2d(line1, nline1);
    if(nline2)free_memory_2d(line2, nline2);
    
    
}
